@@ -33,7 +33,11 @@
 #include "pnfs_utils.h"
 #include "nfs_exports.h"
 #include "sal_data.h"
+#include "sal_functions.h"
 #include "fsal_types.h"
+#ifdef USE_LTTNG
+#include "gsh_lttng/fsal_gluster.h"
+#endif
 
 /* fsal_obj_handle common methods
  */
@@ -66,6 +70,10 @@ static void handle_release(struct fsal_obj_handle *obj_hdl)
 		if (op_ctx && op_ctx->fsal_export) {
 			/* Since handle gets released as part of internal
 			 * operation, we may not need to set credentials */
+#ifdef USE_LTTNG
+			tracepoint(fsalgl, close_fd, __func__, __LINE__,
+				   my_fd->glfd);
+#endif
 			status = glusterfs_close_my_fd(my_fd);
 			if (FSAL_IS_ERROR(status)) {
 				LogCrit(COMPONENT_FSAL,
@@ -81,6 +89,7 @@ static void handle_release(struct fsal_obj_handle *obj_hdl)
 					strerror(errno), errno);
 			}
 		}
+		my_fd->glfd = NULL;
 	}
 
 	if (my_fd->creds.caller_garray) {
@@ -278,12 +287,14 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			  &op_ctx->creds->caller_gid,
 			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+			  op_ctx->creds->caller_garray,
+			  socket_addr(&op_ctx->client->cl_addrbuf),
+			  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 	/** @todo : Can we use globalfd instead */
 	glfd = glfs_h_opendir(glfs_export->gl_fs->fs, objhandle->glhandle);
 
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 	if (glfd == NULL)
 		return gluster2fsal_error(errno);
@@ -300,7 +311,9 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 		SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 				  &op_ctx->creds->caller_gid,
 				  op_ctx->creds->caller_glen,
-				  op_ctx->creds->caller_garray);
+				  op_ctx->creds->caller_garray,
+				  socket_addr(&op_ctx->client->cl_addrbuf),
+				  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 #ifndef USE_GLUSTER_XREADDIRPLUS
 		rc = glfs_readdir_r(glfd, &de, &pde);
@@ -308,7 +321,7 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 		rc = glfs_xreaddirplus_r(glfd, flags, &xstat, &de, &pde);
 #endif
 
-		SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+		SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 		if (rc < 0) {
 			status = gluster2fsal_error(errno);
@@ -420,11 +433,13 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			  &op_ctx->creds->caller_gid,
 			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+			  op_ctx->creds->caller_garray,
+			  socket_addr(&op_ctx->client->cl_addrbuf),
+			  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 	rc = glfs_closedir(glfd);
 
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 	if (rc < 0)
 		status = gluster2fsal_error(errno);
 #ifdef GLTIMING
@@ -463,13 +478,15 @@ static fsal_status_t makedir(struct fsal_obj_handle *dir_hdl,
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			  &op_ctx->creds->caller_gid,
 			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+			  op_ctx->creds->caller_garray,
+			  socket_addr(&op_ctx->client->cl_addrbuf),
+			  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 	glhandle =
 	    glfs_h_mkdir(glfs_export->gl_fs->fs, parenthandle->glhandle, name,
 			 fsal2unix_mode(attrib->mode), &sb);
 
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 	if (glhandle == NULL) {
 		status = gluster2fsal_error(errno);
@@ -589,13 +606,15 @@ static fsal_status_t makenode(struct fsal_obj_handle *dir_hdl,
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			  &op_ctx->creds->caller_gid,
 			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+			  op_ctx->creds->caller_garray,
+			  socket_addr(&op_ctx->client->cl_addrbuf),
+			  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 	glhandle =
 	    glfs_h_mknod(glfs_export->gl_fs->fs, parenthandle->glhandle, name,
 			 create_mode | fsal2unix_mode(attrib->mode), ndev, &sb);
 
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 	if (glhandle == NULL) {
 		status = gluster2fsal_error(errno);
@@ -690,13 +709,15 @@ static fsal_status_t makesymlink(struct fsal_obj_handle *dir_hdl,
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			  &op_ctx->creds->caller_gid,
 			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+			  op_ctx->creds->caller_garray,
+			  socket_addr(&op_ctx->client->cl_addrbuf),
+			  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 	glhandle =
 	    glfs_h_symlink(glfs_export->gl_fs->fs, parenthandle->glhandle, name,
 			   link_path, &sb);
 
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 	if (glhandle == NULL) {
 		status = gluster2fsal_error(errno);
@@ -784,12 +805,14 @@ static fsal_status_t readsymlink(struct fsal_obj_handle *obj_hdl,
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			  &op_ctx->creds->caller_gid,
 			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+			  op_ctx->creds->caller_garray,
+			  socket_addr(&op_ctx->client->cl_addrbuf),
+			  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 	rc = glfs_h_readlink(glfs_export->gl_fs->fs, objhandle->glhandle,
 			     link_content->addr, link_content->len);
 
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 	if (rc < 0) {
 		status = gluster2fsal_error(errno);
@@ -965,12 +988,14 @@ static fsal_status_t linkfile(struct fsal_obj_handle *obj_hdl,
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			  &op_ctx->creds->caller_gid,
 			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+			  op_ctx->creds->caller_garray,
+			  socket_addr(&op_ctx->client->cl_addrbuf),
+			  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 	rc = glfs_h_link(glfs_export->gl_fs->fs, objhandle->glhandle,
 			 dstparenthandle->glhandle, name);
 
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 	if (rc != 0) {
 		status = gluster2fsal_error(errno);
@@ -1014,12 +1039,14 @@ static fsal_status_t renamefile(struct fsal_obj_handle *obj_hdl,
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			  &op_ctx->creds->caller_gid,
 			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+			  op_ctx->creds->caller_garray,
+			  socket_addr(&op_ctx->client->cl_addrbuf),
+			  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 	rc = glfs_h_rename(glfs_export->gl_fs->fs, srcparenthandle->glhandle,
 			   old_name, dstparenthandle->glhandle, new_name);
 
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 	if (rc != 0) {
 		status = gluster2fsal_error(errno);
@@ -1058,12 +1085,14 @@ static fsal_status_t file_unlink(struct fsal_obj_handle *dir_hdl,
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			  &op_ctx->creds->caller_gid,
 			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+			  op_ctx->creds->caller_garray,
+			  socket_addr(&op_ctx->client->cl_addrbuf),
+			  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 	rc = glfs_h_unlink(glfs_export->gl_fs->fs, parenthandle->glhandle,
 			   name);
 
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 	if (rc != 0)
 		status = gluster2fsal_error(errno);
@@ -1087,6 +1116,135 @@ static fsal_status_t share_op(struct fsal_obj_handle *obj_hdl,
 }
 */
 
+/*
+ * @brief: Copy glusterfs_fd structure
+ *
+ * Note: If is_dup is set to true, a new glusterfs_fd is created
+ * with extra ref and hence need to be closed separately.
+ * Whereas if false, both src_fd and dst_fd contain single reference
+ * to glfd. Hence closing one of the them shall destroy other fd too.
+ */
+fsal_status_t glusterfs_copy_my_fd(struct glusterfs_fd *src_fd,
+				 struct glusterfs_fd *dst_fd,
+				 bool is_dup)
+{
+	fsal_status_t status = { ERR_FSAL_NO_ERROR, 0 };
+
+	if (!src_fd || !dst_fd) {
+		return fsalstat(ERR_FSAL_INVAL, 0);
+	}
+
+	if (is_dup) {
+		dst_fd->glfd = glfs_dup(src_fd->glfd);
+		if (src_fd->creds.caller_glen)
+			dst_fd->creds.caller_garray =
+				gsh_memdup(src_fd->creds.caller_garray,
+					   src_fd->creds.caller_glen *
+					   sizeof(gid_t));
+	} else {
+		dst_fd->glfd = src_fd->glfd;
+		dst_fd->creds.caller_garray = src_fd->creds.caller_garray;
+	}
+
+	dst_fd->openflags = src_fd->openflags;
+	dst_fd->creds.caller_uid = src_fd->creds.caller_uid;
+	dst_fd->creds.caller_gid = src_fd->creds.caller_gid;
+	dst_fd->creds.caller_glen = src_fd->creds.caller_glen;
+#ifdef USE_GLUSTER_DELEGATION
+	memcpy(dst_fd->lease_id, src_fd->lease_id, GLAPI_LEASE_ID_SIZE);
+#endif
+	return status;
+}
+
+struct glfs_object*
+glusterfs_create_my_fd(struct glusterfs_handle *parenthandle, const char *name,
+		   fsal_openflags_t openflags, int posix_flags,
+		   mode_t unix_mode, struct stat *sb,
+		   struct glusterfs_fd *my_fd)
+{
+	struct glusterfs_export *glfs_export =
+	    container_of(op_ctx->fsal_export, struct glusterfs_export, export);
+	gid_t **garray_copy = NULL;
+	int p_flags = 0;
+	struct glfs_object *glhandle = NULL;
+#ifdef GLTIMING
+	struct timespec s_time, e_time;
+
+	now(&s_time);
+#endif
+
+	if (!parenthandle || !name || !sb || !my_fd) {
+		errno = EINVAL;
+		return NULL;
+	}
+
+	LogFullDebug(COMPONENT_FSAL,
+		     "my_fd->fd = %p openflags = %x, posix_flags = %x",
+		     my_fd->glfd, openflags, posix_flags);
+
+	assert(my_fd->glfd == NULL
+	       && my_fd->openflags == FSAL_O_CLOSED && openflags != 0);
+
+	LogFullDebug(COMPONENT_FSAL,
+		     "openflags = %x, posix_flags = %x",
+		     openflags, posix_flags);
+
+	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
+			  &op_ctx->creds->caller_gid,
+			  op_ctx->creds->caller_glen,
+			  op_ctx->creds->caller_garray,
+			  socket_addr(&op_ctx->client->cl_addrbuf),
+			  socket_addr_len(&op_ctx->client->cl_addrbuf));
+
+	glhandle = glfs_h_creat_open(glfs_export->gl_fs->fs,
+				 parenthandle->glhandle,
+				 name, p_flags, unix_mode, sb,
+				 &my_fd->glfd);
+
+	/* restore credentials */
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
+
+	if (!glhandle || my_fd->glfd == NULL) {
+		goto out;
+	}
+
+	my_fd->openflags = FSAL_O_NFS_FLAGS(openflags);
+	my_fd->creds.caller_uid = op_ctx->creds->caller_uid;
+	my_fd->creds.caller_gid = op_ctx->creds->caller_gid;
+	my_fd->creds.caller_glen = op_ctx->creds->caller_glen;
+	garray_copy = &my_fd->creds.caller_garray;
+
+	if ((*garray_copy) != NULL) {
+		/* Replace old creds */
+		gsh_free(*garray_copy);
+		*garray_copy = NULL;
+	}
+
+	if (op_ctx->creds->caller_glen) {
+		(*garray_copy) = gsh_malloc(op_ctx->creds->caller_glen
+					    * sizeof(gid_t));
+		memcpy((*garray_copy), op_ctx->creds->caller_garray,
+			op_ctx->creds->caller_glen * sizeof(gid_t));
+	}
+
+#ifdef USE_GLUSTER_DELEGATION
+	if (socket_addr_len(&op_ctx->client->cl_addrbuf)
+						<= GLAPI_LEASE_ID_SIZE) {
+		memcpy(my_fd->lease_id,
+		       socket_addr(&op_ctx->client->cl_addrbuf),
+		       socket_addr_len(&op_ctx->client->cl_addrbuf));
+	} else
+		memset(my_fd->lease_id, 0, GLAPI_LEASE_ID_SIZE);
+#endif
+
+out:
+#ifdef GLTIMING
+	now(&e_time);
+	latency_update(&s_time, &e_time, lat_file_open);
+#endif
+	return glhandle;
+}
+
 fsal_status_t glusterfs_open_my_fd(struct glusterfs_handle *objhandle,
 				   fsal_openflags_t openflags,
 				   int posix_flags,
@@ -1117,13 +1275,15 @@ fsal_status_t glusterfs_open_my_fd(struct glusterfs_handle *objhandle,
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			  &op_ctx->creds->caller_gid,
 			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+			  op_ctx->creds->caller_garray,
+			  socket_addr(&op_ctx->client->cl_addrbuf),
+			  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 	glfd = glfs_h_open(glfs_export->gl_fs->fs, objhandle->glhandle,
 			   posix_flags);
 
 	/* restore credentials */
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 	if (glfd == NULL) {
 		status = gluster2fsal_error(errno);
@@ -1150,6 +1310,16 @@ fsal_status_t glusterfs_open_my_fd(struct glusterfs_handle *objhandle,
 			op_ctx->creds->caller_glen * sizeof(gid_t));
 	}
 
+#ifdef USE_GLUSTER_DELEGATION
+	if (socket_addr_len(&op_ctx->client->cl_addrbuf)
+						<= GLAPI_LEASE_ID_SIZE) {
+		memcpy(my_fd->lease_id,
+		       socket_addr(&op_ctx->client->cl_addrbuf),
+		       socket_addr_len(&op_ctx->client->cl_addrbuf));
+	} else
+		memset(my_fd->lease_id, 0, GLAPI_LEASE_ID_SIZE);
+#endif
+
 out:
 #ifdef GLTIMING
 	now(&e_time);
@@ -1177,12 +1347,18 @@ fsal_status_t glusterfs_close_my_fd(struct glusterfs_fd *my_fd)
 		SET_GLUSTER_CREDS(glfs_export, &my_fd->creds.caller_uid,
 				  &my_fd->creds.caller_gid,
 				  my_fd->creds.caller_glen,
-				  my_fd->creds.caller_garray);
+				  my_fd->creds.caller_garray,
+#ifdef USE_GLUSTER_DELEGATION
+				  my_fd->lease_id,
+				  GLAPI_LEASE_ID_SIZE);
+#else
+				  NULL, 0);
+#endif
 
 		rc = glfs_close(my_fd->glfd);
 
 		/* restore credentials */
-		SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+		SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 		if (rc != 0) {
 			status = gluster2fsal_error(errno);
@@ -1201,6 +1377,9 @@ fsal_status_t glusterfs_close_my_fd(struct glusterfs_fd *my_fd)
 		gsh_free(my_fd->creds.caller_garray);
 		my_fd->creds.caller_garray = NULL;
 	}
+#ifdef USE_GLUSTER_DELEGATION
+	memset(my_fd->lease_id, 0, GLAPI_LEASE_ID_SIZE);
+#endif
 
 #ifdef GLTIMING
 	now(&e_time);
@@ -1264,13 +1443,19 @@ fsal_status_t glusterfs_open_func(struct fsal_obj_handle *obj_hdl,
 {
 	struct glusterfs_handle *myself;
 	int posix_flags = 0;
+	fsal_status_t status = {ERR_FSAL_NO_ERROR, 0};
 
 	myself = container_of(obj_hdl, struct glusterfs_handle, handle);
 
 	fsal2posix_openflags(openflags, &posix_flags);
 
-	return glusterfs_open_my_fd(myself, openflags, posix_flags,
+	status = glusterfs_open_my_fd(myself, openflags, posix_flags,
 				   (struct glusterfs_fd *)fd);
+#ifdef USE_LTTNG
+	tracepoint(fsalgl, open_fd, __func__, __LINE__, posix_flags,
+		   myself->globalfd.glfd);
+#endif
+	return status;
 }
 
 /**
@@ -1349,6 +1534,9 @@ fsal_status_t find_fd(struct glusterfs_fd *my_fd,
 	my_fd->creds.caller_uid = tmp2_fd->creds.caller_uid;
 	my_fd->creds.caller_gid = tmp2_fd->creds.caller_gid;
 	my_fd->creds.caller_glen = tmp2_fd->creds.caller_glen;
+#ifdef USE_GLUSTER_DELEGATION
+	memcpy(my_fd->lease_id, tmp2_fd->lease_id, GLAPI_LEASE_ID_SIZE);
+#endif
 
 out:
 	return status;
@@ -1417,6 +1605,7 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 	    container_of(op_ctx->fsal_export, struct glusterfs_export, export);
 	struct glusterfs_handle *myself, *parenthandle = NULL;
 	struct glusterfs_fd *my_fd = NULL;
+	struct glusterfs_fd tmp_fd = {};
 	struct stat sb = {0};
 	struct glfs_object *glhandle = NULL;
 	unsigned char globjhdl[GFAPI_HANDLE_LENGTH] = {'\0'};
@@ -1426,7 +1615,6 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 	int retval = 0;
 	mode_t unix_mode;
 
-
 #ifdef GLTIMING
 	struct timespec s_time, e_time;
 
@@ -1495,17 +1683,25 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 			/* We need to use the global fd to continue, and take
 			 * the lock to protect it.
 			 */
-			my_fd = &myself->globalfd;
 			PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
+			my_fd = &myself->globalfd;
 		}
 
 		if (my_fd->openflags != FSAL_O_CLOSED) {
+#ifdef USE_LTTNG
+			tracepoint(fsalgl, close_fd, __func__, __LINE__,
+				   my_fd->glfd);
+#endif
 			glusterfs_close_my_fd(my_fd);
 		}
 
 		/* truncate is set in p_flags */
 		status = glusterfs_open_my_fd(myself, openflags, p_flags,
 					      my_fd);
+#ifdef USE_LTTNG
+		tracepoint(fsalgl, open_fd, __func__, __LINE__, p_flags,
+			   my_fd->glfd);
+#endif
 
 		if (FSAL_IS_ERROR(status)) {
 			status = gluster2fsal_error(errno);
@@ -1689,23 +1885,43 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 		 * the caller to perform a permission check.
 		 */
 		*caller_perm_check = true;
-		goto open;
+
+		/* We need to use the global fd to continue, and take
+		 * the lock to protect it.
+		 */
+		PTHREAD_RWLOCK_wrlock(&obj_hdl->obj_lock);
+		if (my_fd == NULL) /* case: state == NULL */
+			my_fd = &myself->globalfd;
+
+		/* De we need to check share reservation conflicts? */
+		status = glusterfs_open_my_fd(myself, openflags,
+					 p_flags, my_fd);
+		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+
+		if (FSAL_IS_ERROR(status))
+			goto direrr;
+		else
+			goto open;
 	}
 
+	if (!my_fd) /* case: state == NULL */
+		my_fd = &tmp_fd;
+
 	/* Become the user because we are creating an object in this dir.
 	 */
 	/* set proper credentials */
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			  &op_ctx->creds->caller_gid,
 			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+			  op_ctx->creds->caller_garray,
+			  socket_addr(&op_ctx->client->cl_addrbuf),
+			  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 	/** @todo: glfs_h_creat doesn't honour NO_CREATE mode. Instead use
 	 *  glfs_h_open to verify if the file already exists.
 	 */
-	glhandle =
-	    glfs_h_creat(glfs_export->gl_fs->fs, parenthandle->glhandle, name,
-			 p_flags, unix_mode, &sb);
+	glhandle = glusterfs_create_my_fd(parenthandle, name, openflags,
+					p_flags, unix_mode, &sb, my_fd);
 
 	if (glhandle == NULL && errno == EEXIST &&
 		 createmode == FSAL_UNCHECKED) {
@@ -1720,17 +1936,17 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 		 * the condition of not wanting to set attributes.
 		 */
 		p_flags &= ~O_EXCL;
-		glhandle =
-		    glfs_h_creat(glfs_export->gl_fs->fs, parenthandle->glhandle,
-				 name, p_flags, unix_mode, &sb);
+		glhandle = glusterfs_create_my_fd(parenthandle, name, openflags,
+					p_flags, unix_mode, &sb, my_fd);
+
 	} else if (!errno) {
 		created = true;
 	}
 
 	/* restore credentials */
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
-	if (glhandle == NULL) {
+	if (glhandle == NULL || my_fd->glfd == NULL) {
 		status = gluster2fsal_error(errno);
 		goto out;
 	}
@@ -1798,17 +2014,34 @@ static fsal_status_t glusterfs_open2(struct fsal_obj_handle *obj_hdl,
 	 * reference to it, and thus we can mamnipulate unlocked which is
 	 * handy since we can then call setattr2 which WILL take the lock
 	 * without a double locking deadlock.
+	 *
+	 * if state != NULL, my_fd contains a valid glfd and hence need to be
+	 * dup'ed to be copied to globalfd. Else my_fd is referring to tmp_fd
+	 * which can safely be copied as is to globalfd.
+	 *
+	 * In case of any further errors, this globalfd gets destroyed as part
+	 * of new_obj->release.
+	 *
 	 */
-	if (my_fd == NULL)
-		my_fd = &myself->globalfd;
 
-open:
-	/* now open it */
-	status = glusterfs_open_my_fd(myself, openflags, p_flags, my_fd);
+	status = glusterfs_copy_my_fd(my_fd, &myself->globalfd,
+				     (state != NULL));
+
+#ifdef USE_LTTNG
+	tracepoint(fsalgl, open_fd, __func__, __LINE__, p_flags,
+		   my_fd->glfd);
+#endif
 
 	if (FSAL_IS_ERROR(status))
-		goto direrr;
+		goto fileerr;
 
+	/* Since we copied it to globalfd, increment
+	 * open_fd_count
+	 */
+	if (state != NULL)
+		(void) atomic_inc_size_t(&open_fd_count);
+
+open:
 	if (created && attrib_set->valid_mask != 0) {
 		/* Set attributes using our newly opened file descriptor as the
 		 * share_fd if there are any left to set (mode and truncate
@@ -1872,6 +2105,11 @@ fileerr:
 	 * FSAL_O_CLOSED flags, it's harmless of closing my_fd twice
 	 * in the floowing obj_ops->release().
 	 */
+
+#ifdef USE_LTTNG
+	tracepoint(fsalgl, close_fd, __func__, __LINE__,
+		   my_fd->glfd);
+#endif
 	glusterfs_close_my_fd(my_fd);
 
 direrr:
@@ -1888,12 +2126,14 @@ direrr:
 		SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 				  &op_ctx->creds->caller_gid,
 				  op_ctx->creds->caller_glen,
-				  op_ctx->creds->caller_garray);
+				  op_ctx->creds->caller_garray,
+				  socket_addr(&op_ctx->client->cl_addrbuf),
+				  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 		glfs_h_unlink(glfs_export->gl_fs->fs, parenthandle->glhandle,
 			      name);
 
-		SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+		SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 	}
 
 
@@ -1962,8 +2202,21 @@ static fsal_status_t glusterfs_reopen2(struct fsal_obj_handle *obj_hdl,
 
 	PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
 
+	/* No need to reopen the file if the openflags are same.
+	 * XXX: Compare creds too post using glfs_h_create_open
+	 * atomic fop to create and open in open2 */
+	if (!(openflags & ~(FSAL_O_OPENFLAGS)) &&
+		(old_openflags == openflags)) {
+		return fsalstat(ERR_FSAL_NO_ERROR, 0);
+	}
+
 	status = glusterfs_open_my_fd(myself, openflags, posix_flags, my_fd);
 
+#ifdef USE_LTTNG
+	tracepoint(fsalgl, open_fd, __func__, __LINE__, posix_flags,
+		   my_fd->glfd);
+#endif
+
 	if (!FSAL_IS_ERROR(status)) {
 		/* Close the existing file descriptor and copy the new
 		 * one over. Make sure no one is using the fd that we are
@@ -1971,7 +2224,12 @@ static fsal_status_t glusterfs_reopen2(struct fsal_obj_handle *obj_hdl,
 		 */
 		PTHREAD_RWLOCK_wrlock(&my_share_fd->fdlock);
 
+#ifdef USE_LTTNG
+		tracepoint(fsalgl, close_fd, __func__, __LINE__,
+			   my_share_fd->glfd);
+#endif
 		glusterfs_close_my_fd(my_share_fd);
+
 		my_share_fd->glfd = my_fd->glfd;
 		my_share_fd->openflags = my_fd->openflags;
 		my_share_fd->creds.caller_uid = my_fd->creds.caller_uid;
@@ -2060,14 +2318,16 @@ static void glusterfs_read2(struct fsal_obj_handle *obj_hdl,
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			  &op_ctx->creds->caller_gid,
 			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+			  op_ctx->creds->caller_garray,
+			  socket_addr(&op_ctx->client->cl_addrbuf),
+			  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 	/* XXX dang switch to preadv_async once async supported */
 	nb_read = glfs_preadv(my_fd.glfd, read_arg->iov, read_arg->iov_count,
 			      seek_descriptor, 0);
 
 	/* restore credentials */
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 	if (seek_descriptor == -1 || nb_read == -1) {
 		retval = errno;
@@ -2104,8 +2364,13 @@ static void glusterfs_read2(struct fsal_obj_handle *obj_hdl,
 	if (glusterfs_fd)
 		PTHREAD_RWLOCK_unlock(&glusterfs_fd->fdlock);
 
-	if (closefd)
+	if (closefd) {
+#ifdef USE_LTTNG
+		tracepoint(fsalgl, close_fd, __func__, __LINE__,
+			   my_fd.glfd);
+#endif
 		glusterfs_close_my_fd(&my_fd);
+	}
 
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
@@ -2133,13 +2398,6 @@ static void glusterfs_write2(struct fsal_obj_handle *obj_hdl,
 	     container_of(op_ctx->fsal_export, struct glusterfs_export, export);
 	struct glusterfs_fd *glusterfs_fd = NULL;
 
-	if (write_arg->info != NULL) {
-		/* Currently we don't support WRITE_PLUS */
-		done_cb(obj_hdl, fsalstat(ERR_FSAL_NOTSUPP, 0), write_arg,
-			caller_arg);
-		return;
-	}
-
 #if 0
 	/** @todo: fsid work */
 	if (obj_hdl->fsal != obj_hdl->fs->fsal) {
@@ -2171,7 +2429,9 @@ static void glusterfs_write2(struct fsal_obj_handle *obj_hdl,
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			  &op_ctx->creds->caller_gid,
 			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+			  op_ctx->creds->caller_garray,
+			  socket_addr(&op_ctx->client->cl_addrbuf),
+			  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 	/* XXX dang switch to pwritev_async once async supported */
 	nb_written = glfs_pwritev(my_fd.glfd, write_arg->iov,
@@ -2179,7 +2439,7 @@ static void glusterfs_write2(struct fsal_obj_handle *obj_hdl,
 				  (write_arg->fsal_stable ? O_SYNC : 0));
 
 	/* restore credentials */
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 	if (nb_written == -1) {
 		retval = errno;
@@ -2194,8 +2454,13 @@ static void glusterfs_write2(struct fsal_obj_handle *obj_hdl,
 	if (glusterfs_fd)
 		PTHREAD_RWLOCK_unlock(&glusterfs_fd->fdlock);
 
-	if (closefd)
+	if (closefd) {
+#ifdef USE_LTTNG
+		tracepoint(fsalgl, close_fd, __func__, __LINE__,
+			   my_fd.glfd);
+#endif
 		glusterfs_close_my_fd(&my_fd);
+	}
 
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
@@ -2254,7 +2519,9 @@ static fsal_status_t seek2(struct fsal_obj_handle *obj_hdl,
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			&op_ctx->creds->caller_gid,
 			op_ctx->creds->caller_glen,
-			op_ctx->creds->caller_garray);
+			op_ctx->creds->caller_garray,
+			socket_addr(&op_ctx->client->cl_addrbuf),
+			socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 	if (info->io_content.what == NFS4_CONTENT_DATA) {
 		what = SEEK_DATA;
@@ -2268,7 +2535,7 @@ static fsal_status_t seek2(struct fsal_obj_handle *obj_hdl,
 	ret = glfs_lseek(my_fd.glfd, offset, what);
 
 	 /* restore credentials */
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 	if (ret < 0) {
 		if (errno == ENXIO) {
@@ -2288,8 +2555,13 @@ static fsal_status_t seek2(struct fsal_obj_handle *obj_hdl,
 	latency_update(&s_time, &e_time, lat_file_seek);
 #endif
 
-	if (closefd)
+	if (closefd) {
+#ifdef USE_LTTNG
+		tracepoint(fsalgl, close_fd, __func__, __LINE__,
+			   my_fd.glfd);
+#endif
 		glusterfs_close_my_fd(&my_fd);
+	}
 
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
@@ -2334,7 +2606,9 @@ static fsal_status_t glusterfs_commit2(struct fsal_obj_handle *obj_hdl,
 		SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 				  &op_ctx->creds->caller_gid,
 				  op_ctx->creds->caller_glen,
-				  op_ctx->creds->caller_garray);
+				  op_ctx->creds->caller_garray,
+				  socket_addr(&op_ctx->client->cl_addrbuf),
+				  socket_addr_len(&op_ctx->client->cl_addrbuf));
 #ifdef USE_GLUSTER_STAT_FETCH_API
 		retval = glfs_fsync(out_fd->glfd, NULL, NULL);
 #else
@@ -2347,11 +2621,16 @@ static fsal_status_t glusterfs_commit2(struct fsal_obj_handle *obj_hdl,
 		}
 
 		/* restore credentials */
-		SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+		SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 	}
 
-	if (closefd)
+	if (closefd) {
+#ifdef USE_LTTNG
+		tracepoint(fsalgl, close_fd, __func__, __LINE__,
+			   out_fd->glfd);
+#endif
 		glusterfs_close_my_fd(out_fd);
+	}
 
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
@@ -2479,10 +2758,16 @@ static fsal_status_t glusterfs_lock_op2(struct fsal_obj_handle *obj_hdl,
 	}
 
 	errno = 0;
-	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
-			  &op_ctx->creds->caller_gid,
-			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+	SET_GLUSTER_CREDS(glfs_export, &my_fd.creds.caller_uid,
+			  &my_fd.creds.caller_gid,
+			  my_fd.creds.caller_glen,
+			  my_fd.creds.caller_garray,
+#ifdef USE_GLUSTER_DELEGATION
+			  my_fd.lease_id,
+			  GLAPI_LEASE_ID_SIZE);
+#else
+			  NULL, 0);
+#endif
 
 	/* Convert lkowner ptr address to opaque string */
 	retval = glfs_fd_set_lkowner(my_fd.glfd, p_owner, sizeof(p_owner));
@@ -2549,13 +2834,18 @@ static fsal_status_t glusterfs_lock_op2(struct fsal_obj_handle *obj_hdl,
 	/* Fall through (retval == 0) */
 
  err:
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 	if (glusterfs_fd)
 		PTHREAD_RWLOCK_unlock(&glusterfs_fd->fdlock);
 
-	if (closefd)
+	if (closefd) {
+#ifdef USE_LTTNG
+		tracepoint(fsalgl, close_fd, __func__, __LINE__,
+			   my_fd.glfd);
+#endif
 		glusterfs_close_my_fd(&my_fd);
+	}
 
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
@@ -2566,6 +2856,127 @@ static fsal_status_t glusterfs_lock_op2(struct fsal_obj_handle *obj_hdl,
 	return status;
 }
 
+#ifdef USE_GLUSTER_DELEGATION
+static fsal_status_t glusterfs_lease_op2(struct fsal_obj_handle *obj_hdl,
+					 struct state_t *state,
+					 void *owner,
+					 fsal_deleg_t deleg)
+{
+	int retval = 0;
+	fsal_status_t status = {ERR_FSAL_NO_ERROR, 0};
+	struct glusterfs_fd my_fd = {0};
+	bool has_lock = false;
+	bool closefd = false;
+	bool bypass = false;
+	fsal_openflags_t openflags = FSAL_O_RDWR;
+	struct glusterfs_handle *myself = NULL;
+	struct glusterfs_fd *glusterfs_fd = NULL;
+	struct glfs_lease lease = {0,};
+	struct glusterfs_export *glfs_export =
+	    container_of(op_ctx->fsal_export, struct glusterfs_export, export);
+
+	myself = container_of(obj_hdl, struct glusterfs_handle, handle);
+
+	switch (deleg) {
+	case FSAL_DELEG_NONE:
+		lease.cmd = GLFS_UNLK_LEASE;
+		openflags = FSAL_O_ANY;
+		/* 'myself' should contain the lease_type obtained.
+		 * If not, we had already unlocked the lease and this is
+		 * duplicate request. Return as noop. */
+		if (myself->lease_type == 0) {
+			LogDebug(COMPONENT_FSAL,
+				"No lease found to unlock");
+			return status;
+		}
+		lease.lease_type = myself->lease_type;
+		break;
+	case FSAL_DELEG_RD:
+		lease.cmd = GLFS_SET_LEASE;
+		openflags = FSAL_O_READ;
+		lease.lease_type = GLFS_RD_LEASE;
+		break;
+	case FSAL_DELEG_WR:
+		lease.cmd = GLFS_SET_LEASE;
+		openflags = FSAL_O_WRITE;
+		lease.lease_type = GLFS_RW_LEASE;
+		break;
+	default:
+		LogCrit(COMPONENT_FSAL, "Unknown requested lease state");
+		return gluster2fsal_error(EINVAL);
+	}
+
+	/* Acquire state's fdlock to prevent OPEN upgrade closing the
+	 * file descriptor while we use it.
+	 */
+	if (state) {
+		glusterfs_fd = &container_of(state, struct glusterfs_state_fd,
+					     state)->glusterfs_fd;
+
+		PTHREAD_RWLOCK_rdlock(&glusterfs_fd->fdlock);
+	}
+
+	/* Get a usable file descriptor */
+	status = find_fd(&my_fd, obj_hdl, bypass, state, openflags,
+			 &has_lock, &closefd, true);
+
+	if (FSAL_IS_ERROR(status)) {
+		LogCrit(COMPONENT_FSAL,
+			"Unable to find fd for lease operation");
+		goto err;
+	}
+
+	/* Since we open unique fd for each NFSv4.x OPEN
+	 * operation, we should have had lease_id set
+	 */
+	memcpy(lease.lease_id, my_fd.lease_id, GLAPI_LEASE_ID_SIZE);
+
+	errno = 0;
+	SET_GLUSTER_CREDS(glfs_export, &my_fd.creds.caller_uid,
+			  &my_fd.creds.caller_gid,
+			  my_fd.creds.caller_glen,
+			  my_fd.creds.caller_garray,
+			  my_fd.lease_id,
+			  GLAPI_LEASE_ID_SIZE);
+	retval = glfs_lease(my_fd.glfd, &lease, NULL, NULL);
+
+	if (retval) {
+		retval = errno;
+		LogWarn(COMPONENT_FSAL, "Unable to %s lease",
+			(deleg == FSAL_DELEG_NONE) ?
+			 "release" : "acquire");
+	} else {
+		if (deleg == FSAL_DELEG_NONE) { /* reset lease_type */
+			myself->lease_type = 0;
+		} else {
+			myself->lease_type = lease.lease_type;
+		}
+	}
+
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
+
+err:
+	if (closefd) {
+#ifdef USE_LTTNG
+		tracepoint(fsalgl, close_fd, __func__, __LINE__,
+			   my_fd.glfd);
+#endif
+		glusterfs_close_my_fd(&my_fd);
+	}
+
+	if (glusterfs_fd)
+		PTHREAD_RWLOCK_unlock(&glusterfs_fd->fdlock);
+
+	if (has_lock)
+		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
+
+	if (retval)
+		status = gluster2fsal_error(retval);
+
+	return status;
+}
+#endif
+
 /**
  * @brief Set attributes on an object
  *
@@ -2653,20 +3064,20 @@ static fsal_status_t glusterfs_setattr2(struct fsal_obj_handle *obj_hdl,
 		SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 				  &op_ctx->creds->caller_gid,
 				  op_ctx->creds->caller_glen,
-				  op_ctx->creds->caller_garray);
+				  op_ctx->creds->caller_garray,
+				  socket_addr(&op_ctx->client->cl_addrbuf),
+				  socket_addr_len(&op_ctx->client->cl_addrbuf));
 #ifdef USE_GLUSTER_STAT_FETCH_API
 		retval = glfs_ftruncate(my_fd.glfd, attrib_set->filesize,
 					NULL, NULL);
 #else
 		retval = glfs_ftruncate(my_fd.glfd, attrib_set->filesize);
 #endif
-		SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+		SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 		if (retval != 0) {
-			if (retval != 0) {
-				status = gluster2fsal_error(errno);
-				goto out;
-			}
+			status = gluster2fsal_error(errno);
+			goto out;
 		}
 	}
 
@@ -2713,7 +3124,7 @@ static fsal_status_t glusterfs_setattr2(struct fsal_obj_handle *obj_hdl,
 
 		retval = clock_gettime(CLOCK_REALTIME, &timestamp);
 		if (retval != 0) {
-			status = gluster2fsal_error(retval);
+			status = gluster2fsal_error(errno);
 			goto out;
 		}
 		buffxstat.buffstat.st_mtim = timestamp;
@@ -2750,7 +3161,9 @@ static fsal_status_t glusterfs_setattr2(struct fsal_obj_handle *obj_hdl,
 	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
 			  &op_ctx->creds->caller_gid,
 			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+			  op_ctx->creds->caller_garray,
+			  socket_addr(&op_ctx->client->cl_addrbuf),
+			  socket_addr_len(&op_ctx->client->cl_addrbuf));
 
 	/* If any stat changed, indicate that */
 	if (mask != 0)
@@ -2797,7 +3210,7 @@ static fsal_status_t glusterfs_setattr2(struct fsal_obj_handle *obj_hdl,
 	}
 
 creds:
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL, NULL, 0);
 
 out:
 	if (FSAL_IS_ERROR(status)) {
@@ -2809,8 +3222,13 @@ out:
 	if (glusterfs_fd)
 		PTHREAD_RWLOCK_unlock(&glusterfs_fd->fdlock);
 
-	if (closefd)
+	if (closefd) {
+#ifdef USE_LTTNG
+		tracepoint(fsalgl, close_fd, __func__, __LINE__,
+			   my_fd.glfd);
+#endif
 		glusterfs_close_my_fd(&my_fd);
+	}
 
 	if (has_lock)
 		PTHREAD_RWLOCK_unlock(&obj_hdl->obj_lock);
@@ -2854,6 +3272,11 @@ static fsal_status_t glusterfs_close2(struct fsal_obj_handle *obj_hdl,
 	 * is operating on the fd while we close it.
 	 */
 	PTHREAD_RWLOCK_wrlock(&my_fd->fdlock);
+
+#ifdef USE_LTTNG
+	tracepoint(fsalgl, close_fd, __func__, __LINE__,
+		   my_fd->glfd);
+#endif
 	status = glusterfs_close_my_fd(my_fd);
 	PTHREAD_RWLOCK_unlock(&my_fd->fdlock);
 
@@ -3085,7 +3508,9 @@ void handle_ops_init(struct fsal_obj_ops *ops)
 	ops->setattr2 = glusterfs_setattr2;
 	ops->close2 = glusterfs_close2;
 	ops->seek2 = seek2;
-
+#ifdef USE_GLUSTER_DELEGATION
+	ops->lease_op2 = glusterfs_lease_op2;
+#endif
 
 	/* pNFS related ops */
 	handle_ops_pnfs(ops);
