@@ -111,6 +111,8 @@ static nfsstat4 ds_read(struct fsal_ds_handle *const ds_pub,
 	rarg.offset = offset;
 	rarg.length = requested_length;
 	rarg.options = 0;
+	if (op_ctx && op_ctx->client)
+		rarg.cli_ip = op_ctx->client->hostaddr_str;
 
 	LogDebug(COMPONENT_PNFS,
 		 "fh len %d type %d key %d: %08x %08x %08x %08x %08x %08x %08x %08x %08x %08x\n",
@@ -185,6 +187,8 @@ static nfsstat4 ds_read_plus(struct fsal_ds_handle *const ds_pub,
 	rarg.length = requested_length;
 	rarg.filesize = &filesize;
 	rarg.options = IO_SKIP_HOLE;
+	if (op_ctx && op_ctx->client)
+		rarg.cli_ip = op_ctx->client->hostaddr_str;
 
 	LogDebug(COMPONENT_PNFS,
 		 "fh len %d type %d key %d: %08x %08x %08x %08x %08x %08x %08x %08x %08x %08x\n",
@@ -284,6 +288,8 @@ static nfsstat4 ds_write(struct fsal_ds_handle *const ds_pub,
 	warg.stability_got = stability_got;
 	warg.verifier4 = (int32_t *) writeverf;
 	warg.options = 0;
+	if (op_ctx && op_ctx->client)
+		warg.cli_ip = op_ctx->client->hostaddr_str;
 
 	LogDebug(COMPONENT_PNFS,
 		 "fh len %d type %d key %d: %08x %08x %08x %08x %08x %08x %08x %08x %08x %08x\n",
