@@ -1,7 +1,7 @@
 /*
  * vim:shiftwidth=8:tabstop=8:
  *
- * Copyright 2017-2018 Red Hat, Inc.
+ * Copyright 2017-2019 Red Hat, Inc.
  * Author: Daniel Gryniewicz  dang@redhat.com
  *
  *
@@ -34,6 +34,13 @@
 
 struct mem_fsal_obj_handle;
 
+enum async_types {
+	MEM_INLINE,
+	MEM_RANDOM_OR_INLINE,
+	MEM_RANDOM,
+	MEM_FIXED,
+};
+
 /**
  * MEM internal export
  */
@@ -50,6 +57,12 @@ struct mem_fsal_export {
 	pthread_rwlock_t mfe_exp_lock;
 	/** List of all the objects in this export */
 	struct glist_head mfe_objs;
+	/** Async delay */
+	uint32_t async_delay;
+	/** Async Stall delay */
+	uint32_t async_stall_delay;
+	/** Type of async */
+	uint32_t async_type;
 };
 
 fsal_status_t mem_lookup_path(struct fsal_export *exp_hdl,
@@ -79,7 +92,6 @@ struct mem_fsal_obj_handle {
 			struct avltree avl_name;
 			struct avltree avl_index;
 			uint32_t numkids;
-			uint32_t next_i; /* next child index */
 		} mh_dir;
 		struct {
 			struct fsal_share share;
@@ -110,7 +122,7 @@ struct mem_dirent {
 	struct mem_fsal_obj_handle *hdl; /**< Handle dirent points to */
 	struct mem_fsal_obj_handle *dir; /**< Dir containing dirent */
 	const char *d_name;		 /**< Name of dirent */
-	uint32_t d_index;		 /**< index in dir */
+	uint64_t d_index;		 /**< index in dir */
 	struct avltree_node avl_n;	 /**< Entry in dir's avl_name tree */
 	struct avltree_node avl_i;	 /**< Entry in dir's avl_index tree */
 	struct glist_head dlist;	 /**< Entry in hdl's dirents list */
@@ -136,6 +148,13 @@ fsal_status_t mem_create_export(struct fsal_module *fsal_hdl,
 				struct config_error_type *err_type,
 				const struct fsal_up_vector *up_ops);
 
+fsal_status_t mem_update_export(struct fsal_module *fsal_hdl,
+				void *parse_node,
+				struct config_error_type *err_type,
+				struct fsal_export *original,
+				struct fsal_module *updated_super);
+
+const char *str_async_type(uint32_t async_type);
 
 #define mem_free_handle(h) _mem_free_handle(h, __func__, __LINE__)
 /**
@@ -181,8 +200,14 @@ struct mem_fsal_module {
 	uint32_t up_interval;
 	/** Next unused inode */
 	uint64_t next_inode;
+	/** Config - number of async threads */
+	uint32_t async_threads;
+	/** Config - whether so use whence-is-name */
+	bool whence_is_name;
 };
 
+/* ASYNC testing */
+extern struct fridgethr *mem_async_fridge;
 
 /* UP testing */
 fsal_status_t mem_up_pkginit(void);
