@@ -76,6 +76,8 @@ static fsal_status_t lookup_int(struct fsal_obj_handle *dir_hdl,
 				const char *path,
 				struct fsal_obj_handle **obj_hdl,
 				struct attrlist *attrs_out,
+				struct stat *rcb_st,
+				uint32_t rcb_st_mask,
 				uint32_t flags)
 {
 	int rc;
@@ -92,10 +94,8 @@ static fsal_status_t lookup_int(struct fsal_obj_handle *dir_hdl,
 	LogFullDebug(COMPONENT_FSAL,
 		"%s enter dir_hdl %p path %s", __func__, dir_hdl, path);
 
-	/* XXX presently, we can only fake attrs--maybe rgw_lookup should
-	 * take struct stat pointer OUT as libcephfs' does */
 	rc = rgw_lookup(export->rgw_fs, dir->rgw_fh, path, &rgw_fh,
-			flags);
+			rcb_st, rcb_st_mask, flags);
 	if (rc < 0)
 		return rgw2fsal_error(rc);
 
@@ -122,7 +122,7 @@ static fsal_status_t lookup(struct fsal_obj_handle *dir_hdl,
 			const char *path, struct fsal_obj_handle **obj_hdl,
 			struct attrlist *attrs_out)
 {
-	return lookup_int(dir_hdl, path, obj_hdl, attrs_out,
+	return lookup_int(dir_hdl, path, obj_hdl, attrs_out, NULL, 0,
 			RGW_LOOKUP_FLAG_NONE);
 }
 
@@ -133,7 +133,8 @@ struct rgw_cb_arg {
 	attrmask_t attrmask;
 };
 
-static bool rgw_cb(const char *name, void *arg, uint64_t offset, uint32_t flags)
+static bool rgw_cb(const char *name, void *arg, uint64_t offset,
+		struct stat *st, uint32_t st_mask, uint32_t flags)
 {
 	struct rgw_cb_arg *rgw_cb_arg = arg;
 	struct fsal_obj_handle *obj = NULL;
@@ -144,11 +145,18 @@ static bool rgw_cb(const char *name, void *arg, uint64_t offset, uint32_t flags)
 	fsal_prepare_attrs(&attrs, rgw_cb_arg->attrmask);
 
 	/* rgw_lookup now accepts type hints */
-	status = lookup_int(rgw_cb_arg->dir_hdl, name, &obj, &attrs,
-			RGW_LOOKUP_FLAG_RCB|
+	status = lookup_int(rgw_cb_arg->dir_hdl, name, &obj, &attrs, st,
+			st_mask, RGW_LOOKUP_FLAG_RCB|
 			(flags & (RGW_LOOKUP_FLAG_DIR|RGW_LOOKUP_FLAG_FILE)));
-	if (FSAL_IS_ERROR(status))
-		return false;
+	if (FSAL_IS_ERROR(status)) {
+		LogWarn(COMPONENT_FSAL,
+			"%s attempt to lookup %s after rgw_readdir() failed "
+			"(%d, %d)",
+			__func__, name, status.major, status.minor);
+		fsal_release_attrs(&attrs);
+		return true; /* no entry is provided to ganesha, so the
+			      * readahead state is unchanged */
+	}
 
 	/** @todo FSF - when rgw gains mark capability, need to change this
 	 *              code...
@@ -857,6 +865,12 @@ fsal_status_t rgw_fsal_open2(struct fsal_obj_handle *obj_hdl,
 			}
 		}
 
+		if (state) {
+			open_state->openflags = FSAL_O_NFS_FLAGS(openflags);
+		} else {
+			handle->openflags = FSAL_O_NFS_FLAGS(openflags);
+		}
+
 		if (createmode >= FSAL_EXCLUSIVE || truncated) {
 			/* refresh attributes */
 			rc = rgw_getattr(export->rgw_fs, handle->rgw_fh, &st,
@@ -1089,7 +1103,11 @@ fsal_status_t rgw_fsal_open2(struct fsal_obj_handle *obj_hdl,
 
 	my_fd->fd = fd;
 #endif
-	handle->openflags = FSAL_O_NFS_FLAGS(openflags);
+	if (state) {
+		open_state->openflags = FSAL_O_NFS_FLAGS(openflags);
+	} else {
+		obj->openflags = FSAL_O_NFS_FLAGS(openflags);
+	}
 
 	*new_obj = &obj->handle;
 
@@ -1397,13 +1415,6 @@ void rgw_fsal_write2(struct fsal_obj_handle *obj_hdl,
 		"%s enter obj_hdl %p state %p", __func__, obj_hdl,
 		write_arg->state);
 
-	if (write_arg->info != NULL) {
-		/* Currently we don't support WRITE_PLUS */
-		done_cb(obj_hdl, fsalstat(ERR_FSAL_NOTSUPP, 0), write_arg,
-			caller_arg);
-		return;
-	}
-
 	/* XXX note no call to fsal_find_fd (or wrapper) */
 
 	for (i = 0; i < write_arg->iov_count; i++) {
@@ -1519,6 +1530,7 @@ fsal_status_t rgw_fsal_close2(struct fsal_obj_handle *obj_hdl,
 	int rc;
 	struct rgw_open_state *open_state;
 	fsal_status_t status;
+	fsal_openflags_t *openflags;
 
 	struct rgw_export *export =
 	    container_of(op_ctx->fsal_export, struct rgw_export, export);
@@ -1550,7 +1562,13 @@ fsal_status_t rgw_fsal_close2(struct fsal_obj_handle *obj_hdl,
 					FSAL_O_CLOSED);
 
 		}
-	} else if (handle->openflags == FSAL_O_CLOSED) {
+
+		openflags = &open_state->openflags;
+	} else {
+		openflags = &handle->openflags;
+	}
+
+	if (unlikely(*openflags == FSAL_O_CLOSED)) {
 		status = fsalstat(ERR_FSAL_NOT_OPENED, 0);
 	} else {
 		rc = rgw_close(export->rgw_fs, handle->rgw_fh,
@@ -1558,7 +1576,7 @@ fsal_status_t rgw_fsal_close2(struct fsal_obj_handle *obj_hdl,
 		if (rc < 0) {
 			status = rgw2fsal_error(rc);
 		} else {
-			handle->openflags = FSAL_O_CLOSED;
+			*openflags = FSAL_O_CLOSED;
 			status = fsalstat(ERR_FSAL_NO_ERROR, 0);
 		}
 	}
