@@ -813,6 +813,7 @@ fsal_status_t mdcache_refresh_attrs(mdcache_entry_t *entry, bool need_acl,
 	struct timespec oldmtime;
 	bool file_deleg = false;
 	cbgetattr_t *cbgetattr;
+	uint32_t original_generation;
 
 	/* Use this to detect if we should invalidate a directory. */
 	oldmtime = entry->attrs.mtime;
@@ -860,6 +861,7 @@ fsal_status_t mdcache_refresh_attrs(mdcache_entry_t *entry, bool need_acl,
 		entry->attrs.request_mask |= ATTR_ACL;
 	}
 
+	original_generation = atomic_fetch_int32_t(&entry->attr_generation);
 	subcall(
 		status = entry->sub_handle->obj_ops->getattrs(
 			entry->sub_handle, &attrs)
@@ -873,6 +875,11 @@ fsal_status_t mdcache_refresh_attrs(mdcache_entry_t *entry, bool need_acl,
 	}
 
 	mdc_update_attr_cache(entry, &attrs);
+	if (atomic_fetch_int32_t(&entry->attr_generation) !=
+		original_generation) {
+		atomic_clear_uint32_t_bits(&entry->mde_flags,
+				MDCACHE_TRUST_ATTRS);
+	}
 
 out:
 	/* Done with the attrs (we didn't need to call this since the
@@ -894,7 +901,7 @@ out:
 		    "attrs ", &entry->attrs, true);
 
 	if (invalidate && entry->obj_handle.type == DIRECTORY &&
-	    gsh_time_cmp(&oldmtime, &entry->attrs.mtime) < 0) {
+	    gsh_time_cmp(&oldmtime, &entry->attrs.mtime) != 0) {
 
 		PTHREAD_RWLOCK_wrlock(&entry->content_lock);
 		mdcache_dirent_invalidate_all(entry);
@@ -1369,13 +1376,16 @@ static fsal_status_t mdcache_merge(struct fsal_obj_handle *orig_hdl,
 
 
 static bool mdcache_is_referral(struct fsal_obj_handle *obj_hdl,
-				struct attrlist *attrs,
+				struct attrlist *unused,
 				bool cache_attrs)
 {
 	mdcache_entry_t *entry =
 		container_of(obj_hdl, mdcache_entry_t, obj_handle);
 	bool result, locked, write_locked;
 	attrmask_t valid_request_mask = 0;
+	struct attrlist attrs[1];
+
+	fsal_prepare_attrs(attrs, ATTR_MODE | ATTR_TYPE);
 
 	PTHREAD_RWLOCK_rdlock(&entry->attr_lock);
 	locked = true;
@@ -1445,6 +1455,7 @@ out:
 		PTHREAD_RWLOCK_unlock(&entry->attr_lock);
 	}
 
+	fsal_release_attrs(attrs);
 	return result;
 }
 
