@@ -451,6 +451,65 @@ lru_insert_chunk(struct dir_chunk *chunk, struct lru_q *q, enum lru_edge edge)
 	QUNLOCK(qlane);
 }
 
+/*
+ * @brief Adjust the order of LRU entries
+ *
+ * @param [in] entry  Entry to adjust.
+ */
+static inline void
+adjust_lru(mdcache_entry_t *entry)
+{
+	mdcache_lru_t *lru = &entry->lru;
+	struct lru_q_lane *qlane = &LRU[lru->lane];
+	struct lru_q *q;
+
+	QLOCK(qlane);
+	switch (lru->qid) {
+	case LRU_ENTRY_L1:
+		q = lru_queue_of(entry);
+		/* advance entry to MRU (of L1) */
+		LRU_DQ_SAFE(lru, q);
+		lru_insert(lru, q, LRU_MRU);
+		break;
+	case LRU_ENTRY_L2:
+		q = lru_queue_of(entry);
+		/* move entry to LRU of L1 */
+		glist_del(&lru->q);     /* skip L1 fixups */
+		--(q->size);
+		q = &qlane->L1;
+		lru_insert(lru, q, LRU_LRU);
+		break;
+	default:
+		/* do nothing */
+		break;
+	}	/* switch qid */
+	QUNLOCK(qlane);
+}
+
+/*
+ * @brief Adjust the order of LRU entries if it is root object of export
+ *
+ * During the entries release process，the root object of export
+ * may block the LRU of LRU queue，we can adjust it to MRU of
+ * LRU and let next entry be released.
+ *
+ * @param [in] entry  Entry to adjust.
+ * @return FSAL status
+ */
+static inline void
+adjust_lru_root_object(mdcache_entry_t *entry)
+{
+	struct lru_q *q = lru_queue_of(entry);
+
+	/* not adjust */
+	if (q->size < 2)
+		return;
+
+	/* adjust export root or junction nodes */
+	if (is_export_pin(&entry->obj_handle))
+		adjust_lru(entry);
+}
+
 /**
  * @brief Clean an entry for recycling.
  *
@@ -660,6 +719,7 @@ lru_reap_impl(enum lru_q_id qid)
 
 		if (unlikely(refcnt != (LRU_SENTINEL_REFCOUNT + 1))) {
 			/* cant use it. */
+			adjust_lru_root_object(entry);
 			mdcache_put(entry);
 			continue;
 		}
@@ -787,6 +847,12 @@ lru_reap_chunk_impl(enum lru_q_id qid, mdcache_entry_t *parent)
 
 		refcnt = atomic_fetch_int32_t(&lru->refcnt);
 		assert(refcnt);
+		if (refcnt != (LRU_SENTINEL_REFCOUNT)) {
+			/* We can't reap a chunk with a ref */
+			QUNLOCK(qlane);
+			continue;
+		}
+
 		/* Get the chunk and parent entry that owns the chunk, all of
 		 * this is valid because we hold the QLANE lock, the chunk was
 		 * in the LRU, and thus the chunk is not yet being destroyed,
@@ -795,39 +861,38 @@ lru_reap_chunk_impl(enum lru_q_id qid, mdcache_entry_t *parent)
 		chunk = container_of(lru, struct dir_chunk, chunk_lru);
 		entry = chunk->parent;
 
-		if (refcnt != (LRU_SENTINEL_REFCOUNT)) {
-			/* We can't reap a chunk with a ref */
-			QUNLOCK(qlane);
-			continue;
-		}
-
-		/* If this chunk belongs to the parent seeking another chunk,
-		 * or if we can get the content_lock for the chunk's parent,
-		 * we can reap this chunk.
+		/* We need entry's content_lock to clean this chunk.
+		 * The usual lock order is content_lock followed by
+		 * chunk QLANE lock. Here we already have chunk QLANE
+		 * lock, so we try to acquire content_lock. If we fail
+		 * to acquire it, we just look for another chunk to
+		 * reap!
+		 *
+		 * Note that the entry is valid but it could be in the
+		 * process of getting destroyed (refcnt could be 0)! If
+		 * we do get the content_lock, it should continue to be
+		 * valid until we release the lock!
+		 *
+		 * If the entry is same as parent, we should already
+		 * have the content lock.
 		 */
 		if (entry == parent ||
 		    pthread_rwlock_trywrlock(&entry->content_lock) == 0) {
-			/* This chunk is eligible for reaping, we can proceed.
-			 */
-			if (entry != parent) {
-				/* We need an LRU ref on parent entry to protect
-				 * it while we do work on it's chunk.
-				 */
-				(void) atomic_inc_int32_t(&entry->lru.refcnt);
-			}
-
 			/* Dequeue the chunk so it won't show up anymore */
 			CHUNK_LRU_DQ_SAFE(lru, lq);
 			chunk->chunk_lru.qid = LRU_ENTRY_NONE;
 
 #ifdef USE_LTTNG
-				tracepoint(mdcache, mdc_lru_reap_chunk,
-					   __func__, __LINE__,
-					   &entry->obj_handle, chunk);
+			tracepoint(mdcache, mdc_lru_reap_chunk,
+				   __func__, __LINE__,
+				   &entry->obj_handle, chunk);
 #endif
 
-			/* Clean the chunk out and indicate the directory is no
-			 * longer completely populated.
+			/* Clean the chunk out and indicate the directory
+			 * is no longer completely populated.  We don't
+			 * need to hold a ref on the entry as we hold its
+			 * content_lock and the chunk is valid under QLANE
+			 * lock.
 			 */
 			mdcache_clean_dirent_chunk(chunk);
 			atomic_clear_uint32_t_bits(&entry->mde_flags,
@@ -839,7 +904,6 @@ lru_reap_chunk_impl(enum lru_q_id qid, mdcache_entry_t *parent)
 				 * acquiring a new chunk for.
 				 */
 				PTHREAD_RWLOCK_unlock(&entry->content_lock);
-				mdcache_put(entry);
 			}
 			QUNLOCK(qlane);
 			return lru;
@@ -1002,18 +1066,13 @@ mdcache_lru_cleanup_try_push(mdcache_entry_t *entry)
 
 	if (cih_latch_entry(&entry->fh_hk.key, &latch, CIH_GET_WLOCK,
 			    __func__, __LINE__)) {
-		uint32_t refcnt;
-
 		QLOCK(qlane);
 
-		refcnt = atomic_fetch_int32_t(&entry->lru.refcnt);
-		/* there are two cases which permit reclaim,
-		 * entry is:
-		 * 1. reachable but unref'd (refcnt==2)
-		 * 2. unreachable, being removed (plus refcnt==0)
-		 *    for safety, take only the former
-		 */
-		if (LRU_ENTRY_RECLAIMABLE(entry, refcnt)) {
+		/* Take the attr lock, so we can check that this entry is still
+		 * not in any export */
+		PTHREAD_RWLOCK_rdlock(&entry->attr_lock);
+
+		if (glist_empty(&entry->export_list)) {
 			/* it worked */
 			struct lru_q *q = lru_queue_of(entry);
 
@@ -1022,17 +1081,18 @@ mdcache_lru_cleanup_try_push(mdcache_entry_t *entry)
 			atomic_set_uint32_t_bits(&entry->lru.flags,
 						 LRU_CLEANUP);
 			/* Note: we didn't take a ref here, so the only ref left
-			 * is the one owned by mdcache_unexport().  When it
-			 * unref's, that will free this object. */
-
-			/* Now we can safely clean out the first_export_id to
-			 * indicate this entry is unmapped.
-			 */
-			atomic_store_int32_t(&entry->first_export_id, -1);
+			 * in this callpath is the one owned by
+			 * mdcache_unexport().  When it unref's, that may free
+			 * this object; otherwise, it will be freed when the
+			 * last op is done, as it's unreachable. */
 
+			/* It's safe to drop the attr lock here, as we have the
+			 * latch, so no one can look up the entry */
+			PTHREAD_RWLOCK_unlock(&entry->attr_lock);
 			QUNLOCK(qlane);
 			cih_remove_latched(entry, &latch, CIH_REMOVE_NONE);
 		} else {
+			PTHREAD_RWLOCK_unlock(&entry->attr_lock);
 			QUNLOCK(qlane);
 		}
 
@@ -1381,6 +1441,34 @@ lru_run(struct fridgethr_context *ctx)
 		}
 	}
 
+	/* We're trying to release the entry cache if the amount
+	 * used is higher than the water level. every time we can
+	 * try best to release the number of entries until entries
+	 * cache below the high water mark. the max number of entries
+	 * released per time is entries_release_size.
+	 */
+	if (lru_state.entries_release_size > 0) {
+		if (lru_state.entries_used > lru_state.entries_hiwat) {
+			size_t released = 0;
+
+			LogFullDebug(COMPONENT_CACHE_INODE_LRU,
+				"Entries used is %" PRIu64
+				" and above water mark, LRU want release %d entries",
+				lru_state.entries_used,
+				lru_state.entries_release_size);
+
+			released = mdcache_lru_release_entries(
+					lru_state.entries_release_size);
+			LogFullDebug(COMPONENT_CACHE_INODE_LRU,
+				"Actually release %zd entries", released);
+		} else {
+			LogFullDebug(COMPONENT_CACHE_INODE_LRU,
+				"Entries used is %" PRIu64
+				" and low water mark: not releasing",
+				lru_state.entries_used);
+		}
+	}
+
 	/* The following calculation will progressively garbage collect
 	 * more frequently as these two factors increase:
 	 * 1. current number of open file descriptors
@@ -1565,30 +1653,40 @@ static void chunk_lru_run(struct fridgethr_context *ctx)
 		 ((uint64_t) new_thread_wait), totalwork);
 }
 
-/**
- * @brief Remove reapable entries until we are below the high-water mark
+/* @brief Release reapable entries until we are below the high-water mark
  *
  * If something refs a lot of entries at the same time, this can put the number
- * of entries above the high water mark.  They will slowly fall, as entries are
- * actually freed, but this may take a very long time.
+ * of entries above the high water mark. Every time we want try best to release
+ * the number of entries, the max number is want_release.
  *
- * This is a big hammer, that will clean up anything it can until either it
- * can't anymore, or we're back below the high water mark.
+ * Normally, want_release equals Entries_Relesase_Size, If it is set to -1
+ * or negative, this going to be a big hammer, that will clean up anything it
+ * can until either it can't anymore, or we're back below the high water mark.
  *
- * @param[in] parm     Parameter description
- * @return Return description
+ * @param[in] want_release Maximum number of entries released. Note that if set
+ *	      negative number, it indicates release all until can't release
+ * @return Return the number of really released
  */
-void lru_cleanup_entries(void)
+size_t mdcache_lru_release_entries(int32_t want_release)
 {
 	mdcache_lru_t *lru;
 	mdcache_entry_t *entry = NULL;
+	size_t released = 0;
+
+	/*release nothing*/
+	if (want_release == 0)
+		return released;
 
 	while ((lru = lru_try_reap_entry())) {
-		if (lru) {
-			entry = container_of(lru, mdcache_entry_t, lru);
-			mdcache_lru_unref(entry);
-		}
+		entry = container_of(lru, mdcache_entry_t, lru);
+		mdcache_lru_unref(entry);
+		++released;
+
+		if (want_release > 0 && released >= want_release)
+			break;
 	}
+
+	return released;
 }
 
 void init_fds_limit(void)
@@ -1721,6 +1819,9 @@ mdcache_lru_pkginit(void)
 	lru_state.entries_hiwat = mdcache_param.entries_hwmark;
 	lru_state.entries_used = 0;
 
+	/* set lru release entries size */
+	lru_state.entries_release_size = mdcache_param.entries_release_size;
+
 	/* Set high and low watermark for chunks.  XXX This seems a
 	   bit fishy, so come back and revisit this. */
 	lru_state.chunks_hiwat = mdcache_param.chunks_hwmark;
@@ -1832,6 +1933,8 @@ mdcache_entry_t *mdcache_lru_get(struct fsal_obj_handle *sub_handle)
 		nentry = alloc_cache_entry();
 	}
 
+	nentry->attr_generation = 0;
+
 	/* Since the entry isn't in a queue, nobody can bump refcnt. */
 	nentry->lru.refcnt = 2;
 	nentry->lru.cf = 0;
@@ -1892,9 +1995,6 @@ fsal_status_t
 _mdcache_lru_ref(mdcache_entry_t *entry, uint32_t flags, const char *func,
 		 int line)
 {
-	mdcache_lru_t *lru = &entry->lru;
-	struct lru_q_lane *qlane = &LRU[lru->lane];
-	struct lru_q *q;
 #ifdef USE_LTTNG
 	int32_t refcnt =
 #endif
@@ -1907,29 +2007,7 @@ _mdcache_lru_ref(mdcache_entry_t *entry, uint32_t flags, const char *func,
 
 	/* adjust LRU on initial refs */
 	if (flags & LRU_REQ_INITIAL) {
-
-		QLOCK(qlane);
-
-		switch (lru->qid) {
-		case LRU_ENTRY_L1:
-			q = lru_queue_of(entry);
-			/* advance entry to MRU (of L1) */
-			LRU_DQ_SAFE(lru, q);
-			lru_insert(lru, q, LRU_MRU);
-			break;
-		case LRU_ENTRY_L2:
-			q = lru_queue_of(entry);
-			/* move entry to LRU of L1 */
-			glist_del(&lru->q);	/* skip L1 fixups */
-			--(q->size);
-			q = &qlane->L1;
-			lru_insert(lru, q, LRU_LRU);
-			break;
-		default:
-			/* do nothing */
-			break;
-		}		/* switch qid */
-		QUNLOCK(qlane);
+		adjust_lru(entry);
 	}			/* initial ref */
 
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
@@ -2196,7 +2274,8 @@ void mdc_lru_map_dirent(mdcache_dir_entry_t *dirent)
 	key.ck = dirent->ck;
 	node = avltree_lookup(&key.node, &exp->dirent_map.map);
 	if (node) {
-		LogFullDebug(COMPONENT_NFS_READDIR, "Already map for %s -> %lx",
+		LogFullDebug(COMPONENT_NFS_READDIR,
+			     "Already map for %s -> %" PRIx64,
 			     dirent->name, dirent->ck);
 		/* Move to MRU */
 		dmap = avltree_container_of(node, mdcache_dmap_entry_t, node);
@@ -2221,7 +2300,7 @@ void mdc_lru_map_dirent(mdcache_dir_entry_t *dirent)
 	dmap->ck = dirent->ck;
 	dmap->name = gsh_strdup(dirent->name);
 	now(&dmap->timestamp);
-	LogFullDebug(COMPONENT_NFS_READDIR, "Mapping %s -> %lx %p:%d",
+	LogFullDebug(COMPONENT_NFS_READDIR, "Mapping %s -> %" PRIx64 " %p:%d",
 		     dmap->name, dmap->ck, exp, exp->dirent_map.count);
 
 	mdc_lru_dirmap_add(exp, dmap);
@@ -2253,7 +2332,7 @@ fsal_cookie_t *mdc_lru_unmap_dirent(uint64_t ck)
 	key.ck = ck;
 	node = avltree_lookup(&key.node, &exp->dirent_map.map);
 	if (!node) {
-		LogFullDebug(COMPONENT_NFS_READDIR, "No map for %lx", ck);
+		LogFullDebug(COMPONENT_NFS_READDIR, "No map for %" PRIx64, ck);
 		PTHREAD_MUTEX_unlock(&exp->dirent_map.mtx);
 		return NULL;
 	}
@@ -2265,8 +2344,8 @@ fsal_cookie_t *mdc_lru_unmap_dirent(uint64_t ck)
 
 	name = dmap->name;
 
-	LogFullDebug(COMPONENT_NFS_READDIR, "Unmapping %s -> %lx", dmap->name,
-		     dmap->ck);
+	LogFullDebug(COMPONENT_NFS_READDIR, "Unmapping %s -> %" PRIx64,
+		     dmap->name, dmap->ck);
 
 	/* Don't free name, we're passing it back to the caller */
 	gsh_free(dmap);
