@@ -103,13 +103,9 @@ static enum load_state {
 
 static void load_fsal_static(const char *name, void (*init)(void))
 {
-	char pname[24];
-	char *dl_path;
+	char *dl_path = gsh_concat("Builtin-", name);
 	struct fsal_module *fsal;
 
-	snprintf(pname, sizeof(pname), "Builtin-%s", name);
-	dl_path = gsh_strdup(pname);
-
 	PTHREAD_MUTEX_lock(&fsal_lock);
 
 	if (load_state != idle)
@@ -198,13 +194,13 @@ int load_fsal(const char *name,
 	char *dl_path;
 	struct fsal_module *fsal;
 	char *bp;
-	char *path = alloca(strlen(nfs_param.core_param.ganesha_modules_loc)
-			    + strlen(name)
-			    + strlen(pathfmt) + 1);
+	size_t size = strlen(nfs_param.core_param.ganesha_modules_loc)
+		      + strlen(name)
+		      + strlen(pathfmt) + 1;
+	char *path = alloca(size);
 
-	sprintf(path, pathfmt,
-		nfs_param.core_param.ganesha_modules_loc,
-		name);
+	(void) snprintf(path, size, pathfmt,
+			nfs_param.core_param.ganesha_modules_loc, name);
 	bp = rindex(path, '/');
 	bp++; /* now it is the basename, lcase it */
 	while (*bp != '\0') {
@@ -492,7 +488,12 @@ void *fsal_init(void *link_mem, void *self_struct)
 	if (link_mem == NULL) {
 		return self_struct; /* NOP */
 	} else if (self_struct == NULL) {
-		return gsh_calloc(1, sizeof(struct fsal_args));
+		void *args = gsh_calloc(1, sizeof(struct fsal_args));
+
+		LogFullDebug(COMPONENT_CONFIG,
+			     "Allocating args %p/%p",
+			     link_mem, args);
+		return args;
 	} else {
 		fp = self_struct;
 		gsh_free(fp->name);
