@@ -58,12 +58,14 @@
 #include "9p.h"
 
 #include <mooshika.h>
+#include <urcu-bp.h>
 
 static void *_9p_rdma_cleanup_conn_thread(void *arg)
 {
 	msk_trans_t *trans = arg;
 	struct _9p_rdma_priv *priv = _9p_rdma_priv_of(trans);
 
+	rcu_register_thread();
 	if (priv) {
 		if (priv->pconn) {
 			LogDebug(COMPONENT_9P,
@@ -93,6 +95,7 @@ static void *_9p_rdma_cleanup_conn_thread(void *arg)
 	}
 
 	msk_destroy_trans(&trans);
+	rcu_unregister_thread();
 	pthread_exit(NULL);
 }
 
@@ -139,6 +142,7 @@ void *_9p_rdma_thread(void *Arg)
 	struct _9p_outqueue *outqueue = trans->private_data;
 	struct sockaddr *addrpeer;
 
+	rcu_register_thread();
 	priv = gsh_calloc(1, sizeof(*priv));
 
 	trans->private_data = priv;
@@ -188,14 +192,14 @@ void *_9p_rdma_thread(void *Arg)
 			 rc);
 		goto error;
 	}
-
+out:
+	rcu_unregister_thread();
 	pthread_exit(NULL);
 
- error:
+error:
 
 	_9p_rdma_cleanup_conn_thread(trans);
-
-	pthread_exit(NULL);
+	goto out;
 }				/* _9p_rdma_handle_trans */
 
 static void _9p_rdma_setup_pernic(msk_trans_t *trans, uint8_t *outrdmabuf)
@@ -326,15 +330,18 @@ void *_9p_rdma_dispatcher_thread(void *Arg)
 	msk_data_t *wdata;
 	struct _9p_outqueue *outqueue;
 
-#define PORT_MAX_LEN 6
-	char port[PORT_MAX_LEN];
+	/* Port is a uint16_t so prints out to at most 5 characters */
+	char port[6];
 
+	rcu_register_thread();
 	memset(&trans_attr, 0, sizeof(msk_trans_attr_t));
 
 	trans_attr.server = _9p_param._9p_rdma_backlog;
 	trans_attr.rq_depth = _9p_param._9p_rdma_inpool_size + 1;
 	trans_attr.sq_depth = _9p_param._9p_rdma_outpool_size + 1;
-	snprintf(port, PORT_MAX_LEN, "%d", _9p_param._9p_rdma_port);
+
+	/* Can't overrun and shouldn't return EOVERFLOW or EINVAL */
+	(void) snprintf(port, sizeof(port), "%d", _9p_param._9p_rdma_port);
 	trans_attr.port = port;
 	trans_attr.node = "::";
 	trans_attr.use_srq = 1;
@@ -419,5 +426,6 @@ void *_9p_rdma_dispatcher_thread(void *Arg)
 		}
 	}			/* for( ;; ) */
 
+	rcu_unregister_thread();
 	pthread_exit(NULL);
 }				/* _9p_rdma_dispatcher_thread */
