@@ -86,7 +86,7 @@ static bool nfs_rpc_cbsim_get_v40_client_ids(DBusMessageIter *args,
 	/* create a reply from the message */
 	now(&ts);
 	dbus_message_iter_init_append(reply, &iter);
-	dbus_append_timestamp(&iter, &ts);
+	gsh_dbus_append_timestamp(&iter, &ts);
 
 	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
 					 DBUS_TYPE_UINT64_AS_STRING, &sub_iter);
@@ -148,7 +148,8 @@ static bool nfs_rpc_cbsim_get_session_ids(DBusMessageIter *args,
 	struct rbt_head *head_rbt;
 	struct hash_data *pdata = NULL;
 	struct rbt_node *pn;
-	char session_id[2 * NFS4_SESSIONID_SIZE];	/* guaranteed to fit */
+	/* guaranteed to fit */
+	char *session_id = alloca(2 * NFS4_SESSIONID_SIZE);
 	nfs41_session_t *session_data;
 	DBusMessageIter iter, sub_iter;
 	struct timespec ts;
@@ -156,7 +157,7 @@ static bool nfs_rpc_cbsim_get_session_ids(DBusMessageIter *args,
 	/* create a reply from the message */
 	now(&ts);
 	dbus_message_iter_init_append(reply, &iter);
-	dbus_append_timestamp(&iter, &ts);
+	gsh_dbus_append_timestamp(&iter, &ts);
 
 	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
 					 DBUS_TYPE_UINT64_AS_STRING, &sub_iter);
@@ -320,8 +321,9 @@ static int cbsim_fake_cbrecall(clientid4 clientid)
 	memset(argop, 0, sizeof(nfs_cb_argop4));
 	argop->argop = NFS4_OP_CB_RECALL;
 	argop->nfs_cb_argop4_u.opcbrecall.stateid.seqid = 0xdeadbeef;
-	strlcpy(argop->nfs_cb_argop4_u.opcbrecall.stateid.other, "0xdeadbeef",
-		12);
+	memcpy(argop->nfs_cb_argop4_u.opcbrecall.stateid.other,
+	       "\0xde\0xad\0xbe\0xef\0xde\0xad\0xbe\0xef\0xde\0xad\0xbe\0xef",
+	       sizeof(argop->nfs_cb_argop4_u.opcbrecall.stateid.other));
 	argop->nfs_cb_argop4_u.opcbrecall.truncate = TRUE;
 	argop->nfs_cb_argop4_u.opcbrecall.fh.nfs_fh4_len = 11;
 	/* leaks, sorry */
