@@ -67,14 +67,8 @@ int nfs3_fsinfo(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	FSINFO3resok * const FSINFO_FIELD =
 		&res->res_fsinfo3.FSINFO3res_u.resok;
 
-	if (isDebug(COMPONENT_NFSPROTO)) {
-		char str[LEN_FH_STR];
-
-		sprint_fhandle3(str, &(arg->arg_fsinfo3.fsroot));
-		LogDebug(COMPONENT_NFSPROTO,
-			 "REQUEST PROCESSING: Calling NFS3_FSINFO handle: %s",
-			 str);
-	}
+	LogNFS3_Operation(COMPONENT_NFSPROTO, req, &arg->arg_fsinfo3.fsroot,
+			  "");
 
 	/* To avoid setting it on each error case */
 	res->res_fsinfo3.FSINFO3res_u.resfail.obj_attributes.attributes_follow =
@@ -121,7 +115,7 @@ int nfs3_fsinfo(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 		     "wtmax = %d | wtpref = %d | wrmult = %d",
 		     FSINFO_FIELD->wtmax, FSINFO_FIELD->wtpref,
 		     FSINFO_FIELD->wtmult);
-	LogFullDebug(COMPONENT_NFSPROTO, "dtpref = %d | maxfilesize = %llu ",
+	LogFullDebug(COMPONENT_NFSPROTO, "dtpref = %d | maxfilesize = %"PRIu64,
 		     FSINFO_FIELD->dtpref, FSINFO_FIELD->maxfilesize);
 
 	/* Allow all kinds of operations to be performed on the server
