@@ -73,6 +73,7 @@ int nfs3_mknod(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	fsal_status_t fsal_status;
 	struct attrlist sattr, attrs;
 	MKNOD3resfail *resfail = &res->res_mknod3.MKNOD3res_u.resfail;
+	MKNOD3resok * const rok = &res->res_mknod3.MKNOD3res_u.resok;
 
 	/* We have the option of not sending attributes, so set ATTR_RDATTR_ERR.
 	 */
@@ -80,17 +81,8 @@ int nfs3_mknod(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 	memset(&sattr, 0, sizeof(sattr));
 
-	if (isDebug(COMPONENT_NFSPROTO)) {
-		char str[LEN_FH_STR];
-		const char *fname;
-
-		fname = (file_name == NULL || *file_name == '\0') ?
-			"<empty name>" : file_name;
-		sprint_fhandle3(str, &(arg->arg_mknod3.where.dir));
-		LogDebug(COMPONENT_NFSPROTO,
-			 "REQUEST PROCESSING: Calling NFS3_MKNOD handle: %s name: %s",
-			 str, fname);
-	}
+	LogNFS3_Operation(COMPONENT_NFSPROTO, req, &arg->arg_mknod3.where.dir,
+			  " name: %s", file_name);
 
 	/* to avoid setting them on each error case */
 	resfail->dir_wcc.before.attributes_follow = FALSE;
@@ -209,8 +201,6 @@ int nfs3_mknod(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	if (FSAL_IS_ERROR(fsal_status))
 		goto out_fail;
 
-	MKNOD3resok * const rok = &res->res_mknod3.MKNOD3res_u.resok;
-
 	/* Build file handle */
 	if (!nfs3_FSALToFhandle(true,
 				&rok->obj.post_op_fh3_u.handle,
@@ -249,10 +239,10 @@ int nfs3_mknod(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 	/* return references */
 	if (parent_obj)
-		parent_obj->obj_ops.put_ref(parent_obj);
+		parent_obj->obj_ops->put_ref(parent_obj);
 
 	if (node_obj)
-		node_obj->obj_ops.put_ref(node_obj);
+		node_obj->obj_ops->put_ref(node_obj);
 
 	return rc;
 }				/* nfs3_mknod */
