@@ -134,8 +134,7 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 			  "");
 
 	/* to avoid setting it on each error case */
-	res->res_readdir3.READDIR3res_u.resfail.dir_attributes.attributes_follow
-		= FALSE;
+	resfail->dir_attributes.attributes_follow = FALSE;
 
 	memset(&tracker, 0, sizeof(tracker));
 
@@ -209,7 +208,7 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	 */
 	if (use_cookie_verifier) {
 		if (attrs_dir.valid_mask == ATTR_RDATTR_ERR) {
-			res->res_readdir3.status = NFS3ERR_SERVERFAULT;
+			res->res_readdirplus3.status = NFS3ERR_SERVERFAULT;
 			LogDebug(COMPONENT_NFS_READDIR,
 				 "Could not fetch ctime");
 			goto out_fail;
@@ -247,12 +246,12 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 	if (begin_cookie == 0) {
 		/* Fill in "." */
-		res->res_readdir3.status =
+		res->res_readdirplus3.status =
 			nfs_readdir_dot_entry(dir_obj, ".", 1,
 					      nfs3_readdirplus_callback,
 					      &tracker, &attrs_dir);
 
-		if (res->res_readdir3.status != NFS3_OK) {
+		if (res->res_readdirplus3.status != NFS3_OK) {
 			rc = NFS_REQ_OK;
 			goto out_destroy;
 		}
@@ -273,7 +272,7 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 			goto out_destroy;
 		}
 
-		res->res_readdir3.status =
+		res->res_readdirplus3.status =
 		    nfs_readdir_dot_entry(parent_dir_obj,
 					  "..",
 					  2,
@@ -283,7 +282,7 @@ int nfs3_readdirplus(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
 		parent_dir_obj->obj_ops->put_ref(parent_dir_obj);
 
-		if (res->res_readdir3.status != NFS3_OK) {
+		if (res->res_readdirplus3.status != NFS3_OK) {
 			rc = NFS_REQ_OK;
 			goto out_destroy;
 		}
