@@ -77,11 +77,8 @@ void nfs4_op_open_CopyRes(OPEN4res *res_dst, OPEN4res *res_src)
  */
 
 static nfsstat4 open4_create_fh(compound_data_t *data,
-				struct fsal_obj_handle *obj,
-				bool state_lock_held)
+				struct fsal_obj_handle *obj)
 {
-	bool set_no_cleanup = false;
-
 	/* Building a new fh */
 	if (!nfs4_FSALToFhandle(false, &data->currentFH, obj,
 					op_ctx->ctx_export)) {
@@ -89,27 +86,12 @@ static nfsstat4 open4_create_fh(compound_data_t *data,
 		return NFS4ERR_SERVERFAULT;
 	}
 
-	if (state_lock_held) {
-		/* Make sure we don't do cleanup holding the state_lock.
-		 * there will be an additional put_ref without the state_lock
-		 * being held. Prevents cleanup in set_current_entry (when
-		 * data->current_obj == obj) and put_ref.
-		 */
-		obj->state_hdl->no_cleanup = true;
-		set_no_cleanup = true;
-	}
-
 	/* Update the current entry */
 	set_current_entry(data, obj);
 
 	/* Put our ref */
 	obj->obj_ops->put_ref(obj);
 
-	if (set_no_cleanup) {
-		/* And clear the no_cleanup we set above. */
-		obj->state_hdl->no_cleanup = false;
-	}
-
 	return NFS4_OK;
 }
 
@@ -317,7 +299,7 @@ bool open4_open_owner(struct nfs_argop4 *op, compound_data_t *data,
 			res_OPEN4->status = nfs4_Errno_status(status);
 			return false;
 		}
-		res_OPEN4->status = open4_create_fh(data, obj_lookup, false);
+		res_OPEN4->status = open4_create_fh(data, obj_lookup);
 	}
 
 	return false;
@@ -384,7 +366,7 @@ static nfsstat4 open4_claim_deleg(OPEN4args *arg, compound_data_t *data)
 		return nfs4_Errno_status(fsal_status);
 	}
 
-	status = open4_create_fh(data, obj_lookup, false);
+	status = open4_create_fh(data, obj_lookup);
 	if (status != NFS4_OK) {
 		LogDebug(COMPONENT_NFS_V4, "open4_create_fh failed");
 		return status;
@@ -911,8 +893,9 @@ static void open4_ex(OPEN4args *arg,
 
 	if (file_obj != NULL) {
 		/* Go ahead and take the state lock now. */
-		PTHREAD_RWLOCK_wrlock(&file_obj->state_hdl->state_lock);
+		STATELOCK_lock(file_obj);
 		state_lock_held = true;
+
 		in_obj = file_obj;
 
 		/* Check if any existing delegations conflict with this open.
@@ -1035,7 +1018,7 @@ static void open4_ex(OPEN4args *arg,
 	if (file_obj == NULL) {
 		/* We have a new cache inode entry, take the state lock. */
 		file_obj = out_obj;
-		PTHREAD_RWLOCK_wrlock(&file_obj->state_hdl->state_lock);
+		STATELOCK_lock(file_obj);
 		state_lock_held = true;
 	}
 
@@ -1098,7 +1081,7 @@ static void open4_ex(OPEN4args *arg,
 		glist_init(&(*file_state)->state_data.share.share_lockstates);
 	}
 
-	res_OPEN4->status = open4_create_fh(data, file_obj, true);
+	res_OPEN4->status = open4_create_fh(data, file_obj);
 
 	if (res_OPEN4->status != NFS4_OK) {
 		if (*new_state) {
@@ -1108,9 +1091,8 @@ static void open4_ex(OPEN4args *arg,
 			*new_state = false;
 		} else {
 			/*Do an open downgrade to the old open flags */
-			status = file_obj->obj_ops->reopen2(file_obj,
-							   *file_state,
-							   old_openflags);
+			status = fsal_reopen2(file_obj, *file_state,
+					      old_openflags, false);
 			if (FSAL_IS_ERROR(status)) {
 				LogCrit(COMPONENT_NFS_V4,
 					"Failed to allocate handle, reopen2 failed with %s",
@@ -1120,7 +1102,7 @@ static void open4_ex(OPEN4args *arg,
 			/* Need to release the state_lock before the put_ref
 			 * call.
 			 */
-			PTHREAD_RWLOCK_unlock(&file_obj->state_hdl->state_lock);
+			STATELOCK_unlock(file_obj);
 			state_lock_held = false;
 
 			/* Release the extra LRU reference on file_obj. */
@@ -1170,7 +1152,7 @@ static void open4_ex(OPEN4args *arg,
 
 	if (!state_lock_held) {
 		/* Acquire state_lock before adding deleg state */
-		PTHREAD_RWLOCK_wrlock(&file_obj->state_hdl->state_lock);
+		STATELOCK_lock(file_obj);
 		state_lock_held = true;
 	}
 
@@ -1180,8 +1162,9 @@ static void open4_ex(OPEN4args *arg,
 	/* Release the attributes (may release an inherited ACL) */
 	fsal_release_attrs(&sattr);
 
-	if (state_lock_held)
-		PTHREAD_RWLOCK_unlock(&file_obj->state_hdl->state_lock);
+	if (state_lock_held) {
+		STATELOCK_unlock(file_obj);
+	}
 
 	if (res_OPEN4->status != NFS4_OK) {
 		/* Cleanup state on error */
