@@ -179,9 +179,9 @@ int nlm_send_async(int proc, state_nlm_client_t *host, void *inarg, void *key)
 				hints.ai_next = NULL;
 
 				/* convert port to string format */
-				sprintf(port_str, "%d",
-					htons(((struct sockaddr_in *)
-						buf->buf)->sin_port));
+				(void) sprintf(port_str, "%d",
+					       htons(((struct sockaddr_in *)
+							buf->buf)->sin_port));
 
 				/* buf with inet is only needed for the port */
 				gsh_free(buf->buf);
@@ -268,11 +268,13 @@ int nlm_send_async(int proc, state_nlm_client_t *host, void *inarg, void *key)
 
 		if (cc->cc_error.re_status == RPC_TIMEDOUT ||
 		    cc->cc_error.re_status == RPC_SUCCESS) {
-			cc->cc_error.re_status = RPC_SUCCESS;
+			retval = RPC_SUCCESS;
 			clnt_req_release(cc);
 			break;
 		}
 
+		retval = cc->cc_error.re_status;
+
 		t = rpc_sperror(&cc->cc_error, "failed");
 		LogCrit(COMPONENT_NLM,
 			"NLM async Client procedure call %d %s",
