@@ -1,7 +1,7 @@
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
- * Copyright 2018-2019 Red Hat, Inc. and/or its affiliates.
+ * Copyright 2018 Red Hat, Inc. and/or its affiliates.
  * Author: Jeff Layton <jlayton@redhat.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -28,6 +28,7 @@
 #include <netdb.h>
 #include <rados/librados.h>
 #include <rados_grace.h>
+#include <urcu-bp.h>
 #include "log.h"
 #include "nfs_core.h"
 #include "sal_functions.h"
@@ -117,8 +118,11 @@ static void rados_cluster_end_grace(void)
 	int ret;
 	rados_write_op_t wop;
 	uint64_t cur, rec;
+	struct gsh_refstr *old_oid;
 
-	if (rados_recov_old_oid[0] == '\0')
+
+	old_oid = rcu_xchg_pointer(&rados_recov_old_oid, NULL);
+	if (!old_oid)
 		return;
 
 	ret = rados_grace_enforcing_off(rados_recov_io_ctx,
@@ -130,13 +134,14 @@ static void rados_cluster_end_grace(void)
 
 	wop = rados_create_write_op();
 	rados_write_op_remove(wop);
-	ret = rados_write_op_operate(wop, rados_recov_io_ctx,
-			       rados_recov_old_oid, NULL, 0);
+	ret = rados_write_op_operate(wop, rados_recov_io_ctx, old_oid->gr_val,
+				     NULL, 0);
 	if (ret)
 		LogEvent(COMPONENT_CLIENTID, "Failed to remove %s: %d",
-			 rados_recov_old_oid, ret);
+			 old_oid->gr_val, ret);
 
-	memset(rados_recov_old_oid, '\0', sizeof(rados_recov_old_oid));
+	synchronize_rcu();
+	gsh_refstr_put(old_oid);
 }
 
 static void rados_cluster_read_clids(nfs_grace_start_t *gsp,
@@ -144,8 +149,10 @@ static void rados_cluster_read_clids(nfs_grace_start_t *gsp,
 				add_rfh_entry_hook add_rfh_entry)
 {
 	int ret;
+	size_t len;
 	uint64_t cur, rec;
 	rados_write_op_t wop;
+	struct gsh_refstr *recov_oid, *old_oid;
 	struct pop_args args = {
 		.add_clid_entry = add_clid_entry,
 		.add_rfh_entry = add_rfh_entry,
@@ -171,23 +178,35 @@ static void rados_cluster_read_clids(nfs_grace_start_t *gsp,
 	 *
 	 * "rec-" followed by epoch in 16 hex digits + nodeid.
 	 */
-	snprintf(rados_recov_oid, sizeof(rados_recov_oid),
-			"rec-%16.16lx:%s", cur, nodeid);
+
+	/* FIXME: assert that rados_recov_oid is NULL? */
+	len = 4 + 16 + 1 + strlen(nodeid) + 1;
+	recov_oid = gsh_refstr_alloc(len);
+
+	/* Can't overrun and shouldn't return EOVERFLOW or EINVAL */
+	(void) snprintf(recov_oid->gr_val, len, "rec-%16.16lx:%s", cur, nodeid);
+	gsh_refstr_get(recov_oid);
+	rcu_set_pointer(&rados_recov_oid, recov_oid);
+
 	wop = rados_create_write_op();
 	rados_write_op_create(wop, LIBRADOS_CREATE_IDEMPOTENT, NULL);
 	rados_write_op_omap_clear(wop);
 	ret = rados_write_op_operate(wop, rados_recov_io_ctx,
-				     rados_recov_oid, NULL, 0);
+				     recov_oid->gr_val, NULL, 0);
+	gsh_refstr_put(recov_oid);
 	rados_release_write_op(wop);
 	if (ret < 0) {
 		LogEvent(COMPONENT_CLIENTID, "Failed to create recovery db");
 		return;
 	};
 
-	snprintf(rados_recov_old_oid, sizeof(rados_recov_old_oid),
-			"rec-%16.16lx:%s", rec, nodeid);
+	old_oid = gsh_refstr_alloc(len);
+
+/* Can't overrun and shouldn't return EOVERFLOW or EINVAL */
+	(void) snprintf(old_oid->gr_val, len, "rec-%16.16lx:%s", rec, nodeid);
+	rcu_set_pointer(&rados_recov_old_oid, old_oid);
 	ret = rados_kv_traverse(rados_ng_pop_clid_entry, &args,
-				rados_recov_old_oid);
+				old_oid->gr_val);
 	if (ret < 0)
 		LogEvent(COMPONENT_CLIENTID,
 			 "Failed to traverse recovery db: %d", ret);
@@ -232,7 +251,6 @@ static void rados_set_client_cb(struct rbt_node *pn, void *arg)
 	nfs_client_id_t *clientid = addr->val.addr;
 	struct rados_cluster_kv_pairs *kvp = arg;
 	char ckey[RADOS_KEY_MAX_LEN];
-	char cval[RADOS_VAL_MAX_LEN];
 
 	/* FIXME: resize arrays in this case? */
 	if (kvp->num >= kvp->slots) {
@@ -240,12 +258,12 @@ static void rados_set_client_cb(struct rbt_node *pn, void *arg)
 		return;
 	}
 
-	rados_kv_create_key(clientid, ckey);
-	rados_kv_create_val(clientid, cval);
+	rados_kv_create_key(clientid, ckey, sizeof(ckey));
+
+	kvp->keys[kvp->num] = gsh_strdup(ckey);
+	kvp->vals[kvp->num] = rados_kv_create_val(clientid,
+						  &kvp->lens[kvp->num]);
 
-	kvp->keys[kvp->num] = strdup(ckey);
-	kvp->vals[kvp->num] = strdup(cval);
-	kvp->lens[kvp->num] = strlen(cval);
 	++kvp->num;
 }
 
@@ -258,9 +276,11 @@ static void rados_set_client_cb(struct rbt_node *pn, void *arg)
 static void rados_cluster_maybe_start_grace(void)
 {
 	int ret, i;
+	size_t len;
 	nfs_grace_start_t gsp = { .event = EVENT_JUST_GRACE };
 	rados_write_op_t wop;
 	uint64_t cur, rec;
+	struct gsh_refstr *recov_oid, *old_oid, *prev_recov_oid;
 	char *keys[RADOS_KV_STARTING_SLOTS];
 	char *vals[RADOS_KV_STARTING_SLOTS];
 	size_t lens[RADOS_KV_STARTING_SLOTS];
@@ -291,11 +311,27 @@ static void rados_cluster_maybe_start_grace(void)
 	 * currently active clients in it.
 	 */
 
-	/* Fix up the oid strings */
-	snprintf(rados_recov_oid, sizeof(rados_recov_oid),
-			"rec-%16.16lx:%s", cur, nodeid);
-	snprintf(rados_recov_old_oid, sizeof(rados_recov_old_oid),
-			"rec-%16.16lx:%s", rec, nodeid);
+	/* Allocate new oid string and xchg it into place */
+	len = 4 + 16 + 1 + strlen(nodeid) + 1;
+	recov_oid = gsh_refstr_alloc(len);
+
+	/* Get an extra working reference of new string */
+	gsh_refstr_get(recov_oid);
+
+	/* Can't overrun and shouldn't return EOVERFLOW or EINVAL */
+	(void) snprintf(recov_oid->gr_val, len, "rec-%16.16lx:%s", cur, nodeid);
+	prev_recov_oid = rcu_xchg_pointer(&rados_recov_oid, recov_oid);
+
+	old_oid = gsh_refstr_alloc(len);
+
+	/* Can't overrun and shouldn't return EOVERFLOW or EINVAL */
+	(void) snprintf(old_oid->gr_val, len, "rec-%16.16lx:%s", rec, nodeid);
+	old_oid = rcu_xchg_pointer(&rados_recov_old_oid, old_oid);
+
+	synchronize_rcu();
+	gsh_refstr_put(prev_recov_oid);
+	if (old_oid)
+		gsh_refstr_put(old_oid);
 
 	/* Populate key/val/len arrays from confirmed client hash */
 	hashtable_for_each(ht_confirmed_client_id, rados_set_client_cb, &kvp);
@@ -308,7 +344,8 @@ static void rados_cluster_maybe_start_grace(void)
 				     (char const * const *)vals,
 				     (const size_t *)lens, kvp.num);
 	ret = rados_write_op_operate(wop, rados_recov_io_ctx,
-				     rados_recov_oid, NULL, 0);
+				     recov_oid->gr_val, NULL, 0);
+	gsh_refstr_put(recov_oid);
 	if (ret)
 		LogEvent(COMPONENT_CLIENTID,
 				"rados_write_op_operate failed: %d", ret);
