@@ -1667,7 +1667,7 @@ void state_complete_grant(state_cookie_entry_t *cookie_entry)
 	 * entry MUST be pinned.
 	 */
 
-	PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
+	STATELOCK_lock(obj);
 
 	/* We need to make sure lock is ready to be granted */
 	if (lock_entry->sle_blocked == STATE_GRANTING) {
@@ -1690,7 +1690,7 @@ void state_complete_grant(state_cookie_entry_t *cookie_entry)
 	 */
 	free_cookie(cookie_entry, true);
 
-	PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
+	STATELOCK_unlock(obj);
 }
 
 /**
@@ -1776,11 +1776,11 @@ void process_blocked_lock_upcall(state_block_data_t *block_data)
 	state_lock_entry_t *lock_entry = block_data->sbd_lock_entry;
 
 	lock_entry_inc_ref(lock_entry);
-	PTHREAD_RWLOCK_wrlock(&lock_entry->sle_obj->state_hdl->state_lock);
+	STATELOCK_lock(lock_entry->sle_obj);
 
 	try_to_grant_lock(lock_entry);
 
-	PTHREAD_RWLOCK_unlock(&lock_entry->sle_obj->state_hdl->state_lock);
+	STATELOCK_unlock(lock_entry->sle_obj);
 	lock_entry_dec_ref(lock_entry);
 }
 
@@ -1981,11 +1981,12 @@ state_status_t state_release_grant(state_cookie_entry_t *cookie_entry)
 	state_lock_entry_t *lock_entry;
 	struct fsal_obj_handle *obj;
 	state_status_t status = STATE_SUCCESS;
+	bool release;
 
 	lock_entry = cookie_entry->sce_lock_entry;
 	obj = cookie_entry->sce_obj;
 
-	PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
+	STATELOCK_lock(obj);
 
 	/* We need to make sure lock is only "granted" once...
 	 * It's (remotely) possible that due to latency, we might end up
@@ -2028,12 +2029,14 @@ state_status_t state_release_grant(state_cookie_entry_t *cookie_entry)
 	/* Check to see if we can grant any blocked locks. */
 	grant_blocked_locks(obj->state_hdl);
 
-	PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
-
 	/* In case all locks have wound up free,
 	 * we must release the object reference.
 	 */
-	if (glist_empty(&obj->state_hdl->file.lock_list))
+	release = glist_empty(&obj->state_hdl->file.lock_list);
+
+	STATELOCK_unlock(obj);
+
+	if (release)
 		obj->obj_ops->put_ref(obj);
 
 
@@ -2239,7 +2242,7 @@ state_status_t state_test(struct fsal_obj_handle *obj,
 
 	LogLock(COMPONENT_STATE, NIV_FULL_DEBUG, "TEST", obj, owner, lock);
 
-	PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
+	STATELOCK_lock(obj);
 
 	found_entry = get_overlapping_entry(obj->state_hdl, owner, lock);
 
@@ -2262,6 +2265,11 @@ state_status_t state_test(struct fsal_obj_handle *obj,
 				"Conflict from FSAL",
 				obj, *holder, conflict);
 			break;
+		case STATE_ESTALE:
+			LogDebug(COMPONENT_STATE,
+				 "Got error %s from FSAL lock operation",
+				 state_err_str(status));
+			break;
 		default:
 			LogMajor(COMPONENT_STATE,
 				 "Got error from FSAL lock operation, error=%s",
@@ -2273,7 +2281,7 @@ state_status_t state_test(struct fsal_obj_handle *obj,
 	if (isFullDebug(COMPONENT_STATE) && isFullDebug(COMPONENT_MEMLEAKS))
 		LogList("Lock List", obj, &obj->state_hdl->file.lock_list);
 
-	PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
+	STATELOCK_unlock(obj);
 
 	return status;
 }
@@ -2623,8 +2631,14 @@ state_status_t state_lock(struct fsal_obj_handle *obj,
 
 		PTHREAD_MUTEX_unlock(&blocked_locks_mutex);
 	} else {
-		LogMajor(COMPONENT_STATE, "Unable to lock FSAL, error=%s",
-			 state_err_str(status));
+		if (status == STATE_ESTALE)
+			LogDebug(COMPONENT_STATE,
+				 "Unable to lock FSAL, error=%s",
+				 state_err_str(status));
+		else
+			LogMajor(COMPONENT_STATE,
+				 "Unable to lock FSAL, error=%s",
+				 state_err_str(status));
 
 		/* Discard lock entry */
 		remove_from_locklist(found_entry);
@@ -2660,7 +2674,7 @@ state_status_t state_unlock(struct fsal_obj_handle *obj,
 		return STATE_BAD_TYPE;
 	}
 
-	PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
+	STATELOCK_lock(obj);
 
 	/* If lock list is empty, there really isn't any work for us to do. */
 	if (glist_empty(&obj->state_hdl->file.lock_list)) {
@@ -2743,7 +2757,7 @@ state_status_t state_unlock(struct fsal_obj_handle *obj,
 		dump_all_locks("All locks (after unlock)");
 
  out_unlock:
-	PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
+	STATELOCK_unlock(obj);
 
 	return status;
 }
@@ -2770,7 +2784,7 @@ state_status_t state_cancel(struct fsal_obj_handle *obj,
 		return STATE_BAD_TYPE;
 	}
 
-	PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
+	STATELOCK_lock(obj);
 
 	/* If lock list is empty, there really isn't any work for us to do. */
 	if (glist_empty(&obj->state_hdl->file.lock_list)) {
@@ -2803,7 +2817,7 @@ state_status_t state_cancel(struct fsal_obj_handle *obj,
 	}
 
  out_unlock:
-	PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
+	STATELOCK_unlock(obj);
 
 	return STATE_SUCCESS;
 }
