@@ -34,6 +34,8 @@
 #include <boost/filesystem/exception.hpp>
 #include <boost/program_options.hpp>
 
+#include "gtest.hh"
+
 extern "C" {
 /* Manually forward this, as 9P is not C++ safe */
 void admin_halt(void);
@@ -47,8 +49,6 @@ void admin_halt(void);
 #include "../FSAL/Stackable_FSALs/FSAL_MDCACHE/mdcache_debug.h"
 }
 
-#include "gtest.hh"
-
 #define TEST_ROOT "readdir_correctness"
 #define TEST_DIR "test_directory"
 #define DIR_COUNT 100000
@@ -124,7 +124,7 @@ namespace {
       }
 
       /* Clean up extra entries */
-      lru_cleanup_entries();
+      mdcache_lru_release_entries(-1);
     }
 
     virtual void TearDown() {
@@ -173,13 +173,6 @@ namespace {
     obj->obj_ops->put_ref(obj);
     return DIR_CONTINUE;
   }
-
-  struct cb_data {
-        u8 *cursor;
-        unsigned int count;
-        unsigned int max;
-  };
-
 } /* namespace */
 
 TEST_F(ReaddirFullCorrectnessTest, BIG)
