@@ -49,13 +49,11 @@ uint64_t nfs_htonl64(uint64_t);
 uint64_t nfs_ntohl64(uint64_t);
 
 /* Error conversion routines */
-nfsstat4 nfs4_Errno_verbose(fsal_errors_t, const char *);
-#define nfs4_Errno(e) nfs4_Errno_verbose(e, __func__)
-#define nfs4_Errno_status(e) nfs4_Errno_verbose(e.major, __func__)
+nfsstat4 nfs4_Errno_verbose(fsal_status_t, const char *);
+#define nfs4_Errno_status(e) nfs4_Errno_verbose(e, __func__)
 #ifdef _USE_NFS3
-nfsstat3 nfs3_Errno_verbose(fsal_errors_t, const char *);
-#define nfs3_Errno(e) nfs3_Errno_verbose(e, __func__)
-#define nfs3_Errno_status(e) nfs3_Errno_verbose(e.major, __func__)
+nfsstat3 nfs3_Errno_verbose(fsal_status_t, const char *);
+#define nfs3_Errno_status(e) nfs3_Errno_verbose(e, __func__)
 #endif
 
 #endif				/* _NFS_CONVERT_H */
