@@ -90,6 +90,7 @@ struct xdr_attrs_args {
 typedef struct fattr4_dent {
 	char *name;		/* The name of the operation */
 	unsigned int supported;	/* Is this action supported? */
+	unsigned int encoded;	/* Can we encode this attribute? */
 	unsigned int size_fattr4; /* The size of the dedicated attr subtype */
 	unsigned int access;	/* The access type for this attributes */
 	attrmask_t attrmask;	/* attr bit for decoding to attrs */
@@ -251,19 +252,39 @@ nfsstat4 nfs4_return_one_state(struct fsal_obj_handle *obj,
 			       size_t body_len, const void *body_val,
 			       bool *deleted);
 
-typedef enum {
-	UTF8_SCAN_NONE = 0,	/* do no validation other than size */
-	UTF8_SCAN_NOSLASH = 1,	/* disallow '/' */
-	UTF8_SCAN_NODOT = 2,	/* disallow '.' and '..' */
-	UTF8_SCAN_CKUTF8 = 4,	/* validate utf8 */
-	UTF8_SCAN_PATH = 8,	/* validate path */
-	UTF8_SCAN_NAME = 3,	/* a name (no embedded /, "." or "..") */
-	UTF8_SCAN_ALL = 7,	/* do the whole thing, name+valid utf8 */
-	UTF8_SCAN_SYMLINK = 12	/* a symlink, allow '/', ".", "..", utf8 */
-} utf8_scantype_t;
+#define UTF8_SCAN_NONE    0x00	/* do no validation other than size */
+#define UTF8_SCAN_NOSLASH 0x01	/* disallow '/' */
+#define UTF8_SCAN_NODOT   0x02	/* disallow '.' and '..' */
+#define UTF8_SCAN_CKUTF8  0x04	/* validate utf8 */
+#define UTF8_SCAN_PATH    0x10	/* validate path length */
 
-nfsstat4 nfs4_utf8string2dynamic(const utf8string *input, utf8_scantype_t scan,
-				 char **obj_name);
+/* Do UTF-8 checking if Enforce_UTF8_Validation is true */
+#define UTF8_SCAN_STRICT \
+	(nfs_param.nfsv4_param.enforce_utf8_vld ? UTF8_SCAN_CKUTF8 : 0)
+
+/* Validate path components, with optional UTF-8 validation */
+#define UTF8_SCAN_PATH_COMP \
+	(UTF8_SCAN_NOSLASH | UTF8_SCAN_NODOT | UTF8_SCAN_STRICT)
+
+nfsstat4 path_filter(const char *name, int scan);
+
+static inline
+nfsstat4 nfs4_utf8string_scan(const utf8string *input, int scan)
+{
+	if (input->utf8string_val == NULL || input->utf8string_len == 0)
+		return NFS4ERR_INVAL;
+
+	if (((scan & UTF8_SCAN_PATH) && input->utf8string_len > MAXPATHLEN) ||
+	    (!(scan & UTF8_SCAN_PATH) && input->utf8string_len > MAXNAMLEN))
+		return NFS4ERR_NAMETOOLONG;
+
+	/* Nothing else to do */
+	if (scan == UTF8_SCAN_NONE || scan == UTF8_SCAN_PATH)
+		return NFS4_OK;
+
+	/* utf8strings are now NUL terminated, so it's ok to call the filter */
+	return path_filter(input->utf8string_val, scan);
+}
 
 int bitmap4_to_attrmask_t(bitmap4 *bitmap4, attrmask_t *mask);
 
@@ -278,9 +299,8 @@ bool nfs4_Fattr_Check_Access_Bitmap(struct bitmap4 *, int);
 bool nfs4_Fattr_Supported(fattr4 *);
 int nfs4_Fattr_cmp(fattr4 *, fattr4 *);
 
-bool nfs3_FSALattr_To_Fattr(struct fsal_obj_handle *obj,
-			    const struct attrlist *FSAL_attr,
-			    fattr3 *Fattr);
+bool nfs3_Fixup_FSALattr(struct fsal_obj_handle *obj,
+			 const struct attrlist *FSAL_attr);
 
 bool is_sticky_bit_set(struct fsal_obj_handle *obj,
 		       const struct attrlist *attr);
@@ -294,6 +314,12 @@ int nfs4_Fattr_To_fsinfo(fsal_dynamicfsinfo_t *, fattr4 *);
 int nfs4_Fattr_Fill_Error(compound_data_t *, fattr4 *, nfsstat4,
 			  struct bitmap4 *, struct xdr_attrs_args *args);
 
+bool xdr_nfs4_fattr_fill_error(XDR *xdrs, struct bitmap4 *req_attrmask,
+			       nfs_cookie4 cookie, component4 *name,
+			       struct xdr_attrs_args *args);
+bool xdr_encode_entry4(XDR *xdrs, struct xdr_attrs_args *args,
+		       struct bitmap4 *req_bitmap, nfs_cookie4 cookie,
+		       component4 *name);
 int nfs4_FSALattr_To_Fattr(struct xdr_attrs_args *, struct bitmap4 *,
 			   fattr4 *);
 
@@ -313,4 +339,6 @@ uint32_t resp_room(compound_data_t *data);
 
 nfsstat4 check_resp_room(compound_data_t *data, uint32_t op_resp_size);
 
+void get_mounted_on_fileid(compound_data_t *data, uint64_t *mounted_on_fileid);
+
 #endif				/* _NFS_PROTO_TOOLS_H */
