@@ -130,7 +130,7 @@ extern hash_table_t *ht_session_id;
 typedef struct nfs41_session_slot__ {
 	sequenceid4 sequence;	/*< Sequence number of this operation */
 	pthread_mutex_t lock;	/*< Lock on the slot */
-	struct COMPOUND4res_extended cached_result;	/*< NFv41: pointer to
+	struct COMPOUND4res_extended *cached_result;	/*< NFv41: pointer to
 							   cached RPC result in
 							   a session's slot */
 } nfs41_session_slot_t;
@@ -329,6 +329,8 @@ struct state_deleg {
 	open_delegation_type4 sd_type;
 	enum deleg_state sd_state;
 	struct cf_deleg_stats sd_clfile_stats;  /* client specific */
+	uint32_t share_access;	/*< The NFSv4 Share Access state */
+	uint32_t share_deny;	/*< The NFSv4 Share Deny state */
 };
 
 /**
@@ -689,7 +691,7 @@ struct nfs_client_id_t {
 			/** Callback identifier */
 			uint32_t cb_callback_ident;
 			/** Universal address */
-			char cb_client_r_addr[SOCK_NAME_MAX + 1];
+			char cb_client_r_addr[SOCK_NAME_MAX];
 			/** Callback program */
 			uint32_t cb_program;
 			bool cb_chan_down;    /* Callback channel state */
@@ -870,6 +872,23 @@ struct file_deleg_stats {
 					   num_opens */
 };
 
+enum cbgetattr_state {
+	CB_GETATTR_NONE = 0, /* initial state or reset to as
+				and when finished processing
+				response */
+	CB_GETATTR_WIP, /* when req sent */
+	CB_GETATTR_RSP_OK, /* successful response */
+	CB_GETATTR_FAILED /* for any failure */
+};
+
+struct cbgetattr_rsp {
+	enum cbgetattr_state state;
+	uint64_t change;
+	uint64_t filesize;
+	bool modified;
+};
+typedef struct cbgetattr_rsp  cbgetattr_t;
+
 /**
  * @brief Per-file state lists
  *
@@ -886,10 +905,14 @@ struct state_file {
 	struct glist_head lock_list;
 	/** Pointers for NLM share list. Protected by state_lock */
 	struct glist_head nlm_share_list;
-	/** true iff write delegated */
+	/** true iff write delegated. Protected by state_lock */
 	bool write_delegated;
+	/** client holding write_deleg. Protected by state_lock */
+	nfs_client_id_t *write_deleg_client;
 	/** Delegation statistics. Protected by state_lock */
 	struct file_deleg_stats fdeleg_stats;
+	/* cbgetattr stats. Protected by state_lock */
+	cbgetattr_t cbgetattr;
 	uint32_t anon_ops;   /* number of anonymous operations
 			      * happening at the moment which
 			      * prevents delegations from being
