@@ -19,9 +19,9 @@
 # Author: Jim Lieb <jlieb@panasas.com>
 #-*- coding: utf-8 -*-
 import sys, time
-from PyQt4.QtCore import *
-from PyQt4 import QtDBus, QtGui
-from PyQt4.QtGui import QColor
+from PyQt5.QtCore import *
+from PyQt5 import QtDBus, QtGui
+from PyQt5.QtGui import QColor
 
 class ExportTableModel(QAbstractTableModel):
     '''
@@ -69,14 +69,14 @@ class ExportTableModel(QAbstractTableModel):
                 self.setData(self.createIndex(i, j), val)
 
     # model abstract methods
-    def setData(self, index, value, role = Qt.EditRole):
+    def setData(self, index, value, role=Qt.EditRole):
         if role == Qt.EditRole:
             row = index.row()
             col = index.column()
-            t = self.exports[row]
+            t_ = self.exports[row]
             if row > self.rowCount() or col > self.columnCount():
                 return False
-            t[col] = value
+            t_[col] = value
             self.emit(SIGNAL('dataChanged'), index, index)
             return True
         return False
