@@ -19,7 +19,7 @@
 # Author: Jim Lieb <jlieb@panasas.com>
 #-*- coding: utf-8 -*-
 
-from PyQt4 import QtCore, QtDBus
+from PyQt5 import QtCore, QtDBus
 from collections import namedtuple
 
 Client = namedtuple('Client',
@@ -49,18 +49,18 @@ class ClientMgr(QtDBus.QDBusAbstractInterface):
         self.show_status = show_status
 
     def AddClient(self, ipaddr):
-        async = self.asyncCall("AddClient", ipaddr)
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("AddClient", ipaddr)
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.clientmgr_done)
 
     def RemoveClient(self, ipaddr):
-        async = self.asyncCall("RemoveClient", ipaddr)
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("RemoveClient", ipaddr)
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.clientmgr_done)
 
     def ShowClients(self):
-        async = self.asyncCall("ShowClients")
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("ShowClients")
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.clientshow_done)
 
     # catch the reply and forward it to the UI
@@ -80,57 +80,55 @@ class ClientMgr(QtDBus.QDBusAbstractInterface):
             self.show_status.emit(False,
                                   "DBUS error:" + str(reply.error().message()))
         else:
-            ts = (reply.argumentAt(0).toPyObject()[0].toULongLong()[0],
-                  reply.argumentAt(0).toPyObject()[1].toULongLong()[0])
-            interval_nsecs = ts[0] * 1000000000 + ts[1]
+            ts_ = (reply.argumentAt(0).toPyObject()[0].toULongLong()[0],
+                   reply.argumentAt(0).toPyObject()[1].toULongLong()[0])
+            interval_nsecs = ts_[0] * 1000000000 + ts_[1]
             clients = []
             for client in reply.argumentAt(1).toPyObject():
-                cl = client.toPyObject()
-                lasttime = cl[9].toPyObject()
-                clt = Client(ClientIP = str(cl[0].toString()),
-                             HasNFSv3 = cl[1].toBool(),
-                             HasMNT = cl[2].toBool(),
-                             HasNLM4 = cl[3].toBool(),
-                             HasRQUOTA = cl[4].toBool(),
-                             HasNFSv40 = cl[5].toBool(),
-                             HasNFSv41 = cl[6].toBool(),
-                             HasNFSv42 = cl[7].toBool(),
-                             Has9P = cl[8].toBool(),
-                             LastTime = (lasttime[0].toPyObject(),
-                                         lasttime[1].toPyObject()))
+                cl_ = client.toPyObject()
+                lasttime = cl_[9].toPyObject()
+                clt = Client(ClientIP=str(cl_[0].toString()),
+                             HasNFSv3=cl_[1].toBool(),
+                             HasMNT=cl_[2].toBool(),
+                             HasNLM4=cl_[3].toBool(),
+                             HasRQUOTA=cl_[4].toBool(),
+                             HasNFSv40=cl_[5].toBool(),
+                             HasNFSv41=cl_[6].toBool(),
+                             HasNFSv42=cl_[7].toBool(),
+                             Has9P=cl_[8].toBool(),
+                             LastTime=(lasttime[0].toPyObject(),
+                                       lasttime[1].toPyObject()))
                 clients.append(clt)
-            self.show_clients.emit(ts, clients)
+            self.show_clients.emit(ts_, clients)
 
 class ClientStats(QtDBus.QDBusAbstractInterface):
     '''
     org.ganesha.nfsd.clientstats
     '''
     def __init__(self, service, path, connection, status_handler, parent=None):
-        super(ClientStats, self).__init__(service,
-                                             path,
-                                             'org.ganesha.nfsd.clientstats',
-                                             connection,
-                                             parent)
+        super(ClientStats, self).__init__(service, path,
+                                          'org.ganesha.nfsd.clientstats',
+                                          connection, parent)
         self.status_handler = status_handler
 
     def GetNFSv3IO(self, ipaddr):
-        async = self.asyncCall("GetNFSv3IO", ipaddr)
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("GetNFSv3IO", ipaddr)
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.io_done)
 
     def GetNFSv40IO(self, ipaddr):
-        async = self.asyncCall("GetNFSv40IO", ipaddr)
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("GetNFSv40IO", ipaddr)
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.io_done)
 
     def GetNFSv41IO(self, ipaddr):
-        async = self.asyncCall("GetNFSv41IO", ipaddr)
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("GetNFSv41IO", ipaddr)
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.io_done)
 
     def GetNFSv41Layouts(self, ipaddr):
-        async = self.asyncCall("GetNFSv41Layouts", ipaddr)
-        status = QtDBus.QDBusPendingCallWatcher(async, self)
+        _async = self.asyncCall("GetNFSv41Layouts", ipaddr)
+        status = QtDBus.QDBusPendingCallWatcher(_async, self)
         status.finished.connect(self.layout_done)
 
     def io_done(self, call):
