@@ -1,12 +1,15 @@
-#!/usr/bin/python2
+#!/usr/bin/python3
 
 # You must initialize the gobject/dbus support for threading
 # before doing anything.
 from __future__ import print_function
-import gobject
+try:
+    import gobject
+except ImportError:
+    from gi.repository import GObject as gobject
 import sys
 
-ipaddr=sys.argv[1]
+ipaddr = sys.argv[1]
 print('event:ip_addr=', ipaddr)
 
 gobject.threads_init()
@@ -20,19 +23,18 @@ bus = dbus.SystemBus()
 
 # Create an object that will proxy for a particular remote object.
 try:
-	admin = bus.get_object("org.ganesha.nfsd",
-        	               "/org/ganesha/nfsd/admin")
+    admin = bus.get_object("org.ganesha.nfsd",
+                           "/org/ganesha/nfsd/admin")
 except dbus.exceptions.DBusException as e:
-	sys.exit("Error: Can't talk to ganesha service on d-bus." \
-		 " Looks like Ganesha is down")
+    sys.exit("Error: Can't talk to ganesha service on d-bus. "
+             "Looks like Ganesha is down")
 
 # call method
-ganesha_grace = admin.get_dbus_method('grace',
-                               'org.ganesha.nfsd.admin')
+ganesha_grace = admin.get_dbus_method('grace', 'org.ganesha.nfsd.admin')
 
 print("Start grace period.")
 
 try:
-	print(ganesha_grace(ipaddr))
+    print(ganesha_grace(ipaddr))
 except dbus.exceptions.DBusException as e:
-	sys.exit("Error: Failed to start grace period")
+    sys.exit("Error: Failed to start grace period")
