@@ -766,7 +766,7 @@ static int fsal_cfg_commit(void *node, void *link_mem, void *self_struct,
 			     "FSAL %s refcount %"PRIu32,
 			     fsal->name,
 			     atomic_fetch_int32_t(&fsal->refcount));
-		err_type->export_ = true;
+		err_type->cur_exp_create_err = true;
 		errcnt++;
 		goto err;
 	}
@@ -868,7 +868,7 @@ static int fsal_update_cfg_commit(void *node, void *link_mem, void *self_struct,
 			     "FSAL %s refcount %"PRIu32,
 			     fsal->name,
 			     atomic_fetch_int32_t(&fsal->refcount));
-		err_type->export_ = true;
+		err_type->cur_exp_create_err = true;
 		errcnt++;
 		goto err;
 	}
@@ -2231,15 +2231,18 @@ fsal_status_t nfs_export_get_root_entry(struct gsh_export *export,
 	if (export->exp_root_obj)
 		export->exp_root_obj->obj_ops->get_ref(export->exp_root_obj);
 
-	PTHREAD_RWLOCK_unlock(&export->lock);
-
 	*obj = export->exp_root_obj;
 
+	PTHREAD_RWLOCK_unlock(&export->lock);
+
 	if (!(*obj))
 		return fsalstat(ERR_FSAL_NOENT, 0);
 
-	if ((*obj)->type != DIRECTORY)
+	if ((*obj)->type != DIRECTORY) {
+		(*obj)->obj_ops->put_ref(*obj);
+		*obj = NULL;
 		return fsalstat(ERR_FSAL_NOTDIR, 0);
+	}
 
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
@@ -2311,6 +2314,28 @@ int init_export_root(struct gsh_export *export)
 	init_root_op_context(&root_op_context, export, export->fsal_export,
 			     0, 0, UNKNOWN_REQUEST);
 
+	/* set expire_time_attr if appropriate */
+	if ((op_ctx->export_perms->set & EXPORT_OPTION_EXPIRE_SET) == 0 &&
+	    (op_ctx->ctx_export->export_perms.set &
+	     EXPORT_OPTION_EXPIRE_SET) != 0) {
+		op_ctx->export_perms->expire_time_attr =
+			op_ctx->ctx_export->export_perms.expire_time_attr;
+		op_ctx->export_perms->set |= EXPORT_OPTION_EXPIRE_SET;
+	}
+	if ((op_ctx->export_perms->set & EXPORT_OPTION_EXPIRE_SET) == 0 &&
+	    (export_opt.conf.set & EXPORT_OPTION_EXPIRE_SET) != 0) {
+		op_ctx->export_perms->expire_time_attr =
+			export_opt.conf.expire_time_attr;
+		op_ctx->export_perms->set |= EXPORT_OPTION_EXPIRE_SET;
+	}
+	if ((op_ctx->export_perms->set & EXPORT_OPTION_EXPIRE_SET) == 0)
+		op_ctx->export_perms->expire_time_attr =
+					export_opt.def.expire_time_attr;
+
+	/* set the EXPORT_OPTION_EXPIRE_SET bit from the export
+	 * into the op_ctx */
+	op_ctx->export_perms->options |= EXPORT_OPTION_EXPIRE_SET;
+
 	/* Lookup for the FSAL Path */
 	LogDebug(COMPONENT_EXPORT,
 		 "About to lookup_path for ExportId=%u Path=%s",
@@ -2356,6 +2381,7 @@ int init_export_root(struct gsh_export *export)
 	PTHREAD_RWLOCK_wrlock(&export->lock);
 
 	/* Pass ref off to export */
+	export_root_object_get(obj);
 	export->exp_root_obj = obj;
 	glist_add_tail(&obj->state_hdl->dir.export_roots,
 		       &export->exp_root_list);
@@ -2411,6 +2437,7 @@ static void release_export(struct gsh_export *export)
 	PTHREAD_RWLOCK_wrlock(&export->lock);
 
 	glist_del(&export->exp_root_list);
+	export_root_object_put(export->exp_root_obj);
 	export->exp_root_obj->obj_ops->put_ref(export->exp_root_obj);
 	export->exp_root_obj = NULL;
 
