@@ -180,6 +180,8 @@ static struct config_item core_params[] = {
 		       nfs_core_param, enable_FULLV4STATS),
 	CONF_ITEM_BOOL("Enable_AUTH_Stats", false,
 		       nfs_core_param, enable_AUTHSTATS),
+	CONF_ITEM_BOOL("Enable_CLNT_AllOps_Stats", false,
+		       nfs_core_param, enable_CLNTALLSTATS),
 	CONF_ITEM_BOOL("Short_File_Handle", false,
 		       nfs_core_param, short_file_handle),
 	CONF_ITEM_I64("Manage_Gids_Expiration", 0, 7*24*60*60, 30*60,
@@ -196,6 +198,8 @@ static struct config_item core_params[] = {
 		       nfs_core_param, enable_UDP),
 	CONF_ITEM_STR("Dbus_Name_Prefix", 1, 255, NULL,
 		       nfs_core_param, dbus_name_prefix),
+	CONF_ITEM_UI32("Max_Uid_To_Group_Reqs", 0, INT32_MAX, 0,
+		       nfs_core_param, max_uid_to_grp_reqs),
 	CONFIG_EOL
 };
 
