@@ -38,11 +38,11 @@ char usage[] =
 	"       ml_glusterfs_client -v volname -g glusterserver -x script [-q] [-d] [-c path]\n"
 	"       ml_glusterfs_client -v volname -g glusterserver [-q] [-d] [-c path]\n"
 	"  ml_glusterfs_client may be run in three modes\n"
-	"  - In the first mode, the client will be driven by a master.\n"
+	"  - In the first mode, the client will be driven by a console.\n"
 	"  - In the second mode, the client is driven by a script.\n"
 	"  - In the third mode, the client interractive.\n"
-	"  -s server      - specify the master's hostname or IP address\n"
-	"  -p port        - specify the master's port number\n"
+	"  -s server      - specify the console's hostname or IP address\n"
+	"  -p port        - specify the console's port number\n"
 	"  -n name        - specify the client's name\n"
 	"  -x script      - specify the name of a script to execute\n"
 	"  -q             - specify quiet mode\n"
@@ -381,9 +381,12 @@ void do_read(struct response *resp)
 
 	if (resp->r_length > MAXSTR)
 		resp->r_length = MAXSTR;
-
+#ifdef USE_GLUSTER_STAT_FETCH_API
+	rc = glfs_read(fds[resp->r_fpos], resp->r_data, resp->r_length,
+			0, NULL);
+#else
 	rc = glfs_read(fds[resp->r_fpos], resp->r_data, resp->r_length, 0);
-
+#endif
 	if (rc < 0) {
 		resp->r_status = STATUS_ERRNO;
 		resp->r_errno = -rc;
